/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid;

import java.util.ArrayList;
import java.util.Iterator;

public class DirtManagerImpl 
	implements gvid.DirtManager {
 
	// should this be a weak or soft reference?
	private DirtManager m_owner;
	private ArrayList m_dirtListeners = new ArrayList();
	private boolean m_bDirty = false;
    
    public DirtManagerImpl( DirtManager owner ) {
    	m_owner = owner;
    }
    
    public void addDirtListener(DirtListener l) {
    	synchronized( m_dirtListeners ) {
			m_dirtListeners.add(l);
    	}
    }

    public void removeDirtListener(DirtListener l) {
    	synchronized( m_dirtListeners ) {
			m_dirtListeners.remove(l);
    	}
    }
    
    public boolean isDirty() {
        return m_bDirty;
    }

    public void setDirty() {
		if( !m_bDirty ) {
			m_bDirty = true;
			notifyDirty();
		}
    }

    public void clearDirty() {
		if( m_bDirty ) {
			m_bDirty = false;
			notifyDirty();
		}
    }
    
    public void copyListeners( DirtManagerImpl impl ) {
    	m_dirtListeners.addAll(impl.m_dirtListeners);
    }

	private void notifyDirty() {
		Object[] listeners = null;
    	synchronized( m_dirtListeners ) {
    		listeners = m_dirtListeners.toArray();
    	}
    	for( int i=0; i < listeners.length; i++ ) {
    		DirtListener l = (DirtListener)listeners[i];
    		l.dirtyChanged( isDirty(), m_owner );
    	}
	}
}