/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.project;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.*;
import gvid.*;
import gvid.ui.*;

public class ProjectWindow 
	extends 	javax.swing.JFrame
	implements	DirtListener {
	ProjectModel m_project = null;
	FilelistPanel m_pane = new FilelistPanel();
	
	public ProjectWindow() {
		setProject(null);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add( m_pane );
	}
	
	public ProjectModel getProject() {
		return m_project;
	}
	
	public boolean newProject(File projectFile) {
		boolean rc = false;
		
		if( closeProject() && projectFile != null ) {
			if( setProject( new ProjectModel( projectFile ) ) ) {
				if( saveProject() ) {				
					rc = true;
				} else {
					setProject(null);
				}
			}
		}
		return rc;
	}
	
	public boolean saveProject() {
		boolean rc = false;
		
		if( m_project != null ) {
			File file = m_project.getProjectFile();
			boolean bContinue = true;
			while( bContinue ) {
				try {
					FileOutputStream fos = new FileOutputStream(file);
					ObjectOutputStream out = new ObjectOutputStream(fos);
					out.writeObject(m_project);
					out.close();
					m_project.clearDirty();
					bContinue = false;
					rc = true;
				} catch( IOException e ) {
					// TODO : Handle write protected files
					System.err.println(e);
					bContinue = false;
				}
			}
		}
		
		return rc;
	}
	
	public boolean openProject( File file ) {
		boolean rc = false;
		if( closeProject() ) {
			try {
				FileInputStream fis = new FileInputStream(file);
				ObjectInputStream in = new ObjectInputStream(fis);
				try {
					ProjectModel project = (ProjectModel)in.readObject();
					project.setProjectFile(file);
					project.clearDirty();
					setProject(project);
					rc = true;
				} catch( OptionalDataException e ) {
					JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "The project file appears to be corrupt.");
				} catch( ClassNotFoundException e ) {
					JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "This does not appear to be a valid project file.");
				} catch( ClassCastException e ) {
					JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "This does not appear to be a valid project file.");
				}
			} catch( FileNotFoundException e ) {
				JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "The project file could not be found.");
			} catch( StreamCorruptedException e ) {
				JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "An error occurred while opening the file.");
			} catch( IOException e ) {
				JOptionPane.showMessageDialog(MainFrame.getMainFrame(), "An error occurred while opening the file.");
			}
		}
		return rc;
	}
	
	public boolean closeProject() {
		boolean rc = false;
		
		if( conditionalSaveProject() ) {
			setProject(null);
			rc = true;
		}
		
		return rc;
	}
	
	public boolean conditionalSaveProject() {
		boolean rc = false;
		if( m_project != null && m_project.isDirty() ) {
			int saveConfirm = JOptionPane.showConfirmDialog(this, "The project has changed since the last save. Would you like to save it now?", "Save Project", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE );
			if( saveConfirm == JOptionPane.YES_OPTION ) {
				rc = saveProject();
			} else if( saveConfirm == JOptionPane.NO_OPTION ) {
				rc = true;
			}
		} else {
			rc = true;
		}
		return rc;
	}
	
	private boolean setProject( ProjectModel project ) {
		if( m_project != null ) {
			m_project.removeDirtListener(this);
		}
		m_project = project;
		if( m_project != null ) {
			m_project.addDirtListener(this);
		}
		m_pane.setProjectModel(m_project);
		buildTitle();
		return true;
	}
	
	private void buildTitle() {
		StringBuffer title = new StringBuffer();
		title.append("Project - ");
		if( m_project != null ) {
			title.append(m_project.getName());
			if( m_project.isDirty() ) {
				title.append("*");
			}
		} else {
			title.append("none");
		}
		setTitle(title.toString());
	}
///////////////////////////////////////////////////////////////////////////////
//	DirtListener	
	public void dirtyChanged( boolean bDirty, DirtManager mgr ) {
		buildTitle();
	}
//	DirtListener	
///////////////////////////////////////////////////////////////////////////////
}