/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import gvid.ui.splitscrollpane.*;
import gvid.ui.editor.*;
import gvid.ui.menu.*;
import gvid.document.*;
import gvid.*;
import gvid.command.*;
import java.util.*;
import gvid.ui.status.*;

public class InternalTextFrame 
	extends 	javax.swing.JInternalFrame
	implements	DirtListener,
				FocusListener, 
				PropertyChangeListener,
				SplitScrollListener,
				StatusSource,
				StatusListener {
	
	SplitScrollPane m_splitPane = null;
	SourceCodeView m_view = null;
	private SourceCodeModel m_document = null;
	String m_title = null;

	public InternalTextFrame( String title ) {
		super(title, true, true, true, true);
		m_title = title;
		m_document = new SourceCodeModel();
		setup();
	}
	
	public InternalTextFrame( File file )
		throws IOException {
		super(file.getCanonicalPath(), true, true, true, true);
		m_document = new SourceCodeModel(file);
		setup();
	}
	
	public StatusSource getStatusSource() {
		return this;
	}
	
	public void setFile( File file ) {
		m_document.setFile(file);
		m_title = null;
		try {
			setTitle( file.getCanonicalPath() );
		} catch( IOException e ) {
			setTitle( "" );
		}		
	}
	
	public File getFile() {
		return m_document.getFile();
	}
	
	public void setup() {
//		long t1 = 0;
//		long t2 = 0;
		getContentPane().setLayout(new BorderLayout());
		
//		t1 = System.currentTimeMillis();
		m_view = new SourceCodeView();
//		t2 = System.currentTimeMillis();
//		System.err.println("new SourceCodeView : "+(t2-t1)+" ms");
		
//		t1 = System.currentTimeMillis();
		m_splitPane = new SplitScrollPane();
//		t2 = System.currentTimeMillis();
//		System.err.println("new SplitScrollPane : "+(t2-t1)+" ms");
		
		m_view.setModel(m_document);
		m_splitPane.setView(m_view);
//		setContentPane(m_splitPane);
		getContentPane().add(m_splitPane, BorderLayout.CENTER);
		
		m_view.addDirtListener(this);
//		m_view.requestDefaultFocus();
		addFocusListener(this);
		addPropertyChangeListener(this);
		m_splitPane.addSplitListener(this);
	}
	
	public EditCommands getEditCommands() {
		return getActiveView().getEditCommands();
	}
	
	public SourceCodeView getActiveView() {
		return (SourceCodeView)(m_splitPane.getActiveView().asComponent());
	}
	
	public SourceCodeModel getDocument() {
		return m_document;
	}
	
	public boolean isUnnamed() {
		return m_document.getFile()==null;
	}	
	
///////////////////////////////////////////////////////////////////////////////
//	DirtListener
	public void dirtyChanged( boolean bDirty, DirtManager mgr ) {
		if( bDirty ) {
			if( m_title != null ) {
				setTitle(m_title+"*");
			} else {
				try {
					setTitle( m_document.getFile().getCanonicalPath() +"*" );
				} catch( Exception e ) {
					setTitle( "*" );
				}
			}
		} else {
			if( m_title != null ) {
				setTitle(m_title);
			} else {
				try {
					setTitle( m_document.getFile().getCanonicalPath() );
				} catch( Exception e ) {
					setTitle( "" );
				}
			}
		}
	}
//	DirtListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	FocusListener
	public void focusGained(FocusEvent e) {
		m_splitPane.requestFocus();
	}
	
	public void focusLost(FocusEvent e) {
	}
//	FocusListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	PropertyChangeListener
	public void propertyChange(PropertyChangeEvent evt) {
		if( evt.getPropertyName().equals("selected") && evt.getNewValue().equals(Boolean.TRUE) ) {
			m_splitPane.requestFocus();
		}
	}
//	PropertyChangeListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	SplitScrollListener
	public void activePaneChanged() {
		SplitScrollView view = m_splitPane.getActiveView();
		if( view instanceof StatusSource ) {
			updateStatusSource((StatusSource)view);
		}
	}
//	SplitScrollListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	StatusSource
	Vector m_statusListeners = new Vector();
	
	StatusSource m_statusSource = null;
	private void updateStatusSource( StatusSource src ) {
		if( src != m_statusSource ) {
			int oldCount = 0;
			if( m_statusSource != null ) {
				m_statusSource.removeStatusListener(this);
				oldCount = m_statusSource.getStatusCount();
			}
			int newCount = 0;
			if( src != null ) {
				src.addStatusListener(this);
				newCount = src.getStatusCount();
			}
			m_statusSource = src;
			if( oldCount == newCount && m_statusSource != null ) {
				notifyStatusMsg();
				for( int i=m_statusSource.getStatusCount(); i > 0; i-- ) {
					notifyStatus(i-1, m_statusSource.getStatusString(i-1), m_statusSource.isStatusActive(i-1));
				}
			} else {
				notifyStatus();
			}
		}
	}
	
	private void notifyStatusMsg() {
		Enumeration enum = m_statusListeners.elements();
		while( enum.hasMoreElements() ) {
			StatusListener l = (StatusListener)enum.nextElement();
			l.statusMessageChanged();
		}
	}
	
	private void notifyStatus(int index, String str, boolean active) {
		Enumeration enum = m_statusListeners.elements();
		while( enum.hasMoreElements() ) {
			StatusListener l = (StatusListener)enum.nextElement();
			l.statusChanged(index, str, active);
		}
	}
	
	private void notifyStatus() {
		Enumeration enum = m_statusListeners.elements();
		while( enum.hasMoreElements() ) {
			StatusListener l = (StatusListener)enum.nextElement();
			l.statusChanged();
		}
	}
	
	public void addStatusListener( StatusListener l ) {
		m_statusListeners.addElement(l);
	}
	public void removeStatusListener( StatusListener l ) {
		m_statusListeners.removeElement(l);
	}
	public String getStatusMsg() {
		String rc = "";
		if( m_statusSource != null ) {
			rc = m_statusSource.getStatusMsg();
		}
		return rc;
	}
	public void setStatusMsg( String str ) {
		if( m_statusSource != null ) {
			m_statusSource.setStatusMsg(str);
		}
	}
	public int getStatusCount() {
		int rc = 0;
		if( m_statusSource != null ) {
			rc = m_statusSource.getStatusCount();
		}
		return rc;
	}
	public String getStatusString(int index) {
		String rc = "";
		if( m_statusSource != null ) {
			rc = m_statusSource.getStatusString(index);
		}
		return rc;
	}
	public boolean isStatusActive(int index) {
		boolean rc = false;
		if( m_statusSource != null ) {
			rc = m_statusSource.isStatusActive(index);
		}
		return rc;
	}	
//	StatusSource
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	StatusListener
	public void statusMessageChanged() {
		notifyStatusMsg();
	}

	public void statusChanged( int index, String str, boolean active ) {
		notifyStatus( index, str, active );
	}
	
	public void statusChanged() {
		notifyStatus();
	}
//	StatusListener
///////////////////////////////////////////////////////////////////////////////
}