/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import gvid.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;
import java.io.*;

public class MainFileCommandsImpl 
	implements gvid.command.FileCommands {
	MainFrame m_mainFrame = null;
		
	public MainFileCommandsImpl(MainFrame frame) {
        m_mainFrame = frame;  
    }
	
	public int getCommandGroup() {
		return FILEGROUP;
	}	
	
	public boolean isAvailable( int cmd ) {
		boolean rc = false;
		
		// TODO: Implement isAvailable
		rc = true;
		
		return rc;
	}
	
	public void executeCommand(int cmd) {
		switch( cmd ) {
			case APPLICATIONEXIT: {
				applicationExit();		
			} break;
			case FILECLOSE: {
				fileClose();			
			} break;
			case FILEGOTO: {
				fileGoTo();				
			} break;
			case FILEOPEN: {
				fileOpen(); 			
			} break;
			case FILEPRINT: {
				filePrint();			
			} break;
			case FILEPRINTPAGESETUP: {
				filePrintPageSetup();
			} break;
			case FILESAVE: {
				fileSave();				
			} break;
			case FILESAVEALL: {
				fileSaveAll();			
			} break;
			case FILESAVEALLEXIT: {
				fileSaveAllExit();		
			} break;
			case FILESAVEAS: {
				fileSaveAs();			
			} break;
			case NEWITEM: {
				newItem();				
			} break;
			case NEWTEXT: {
				newText();				
			} break;
			case WORKSPACECLOSE: {
				workspaceClose();		
			} break;
			case WORKSPACEOPEN: {
				workspaceOpen();		
			} break;
			case WORKSPACESAVE: {
				workspaceSave();		
			} break;
			case WINDOWCLOSEALL: {
				windowCloseAll();
			} break;
			case WINDOWNEXT: {
				windowNext();
			} break;
			case WINDOWPREV: {
				windowPrev();
			} break;
			case WINDOWCASCADE: {
				windowCascade();
			} break;
			case WINDOWTILEHORIZONTAL: {
				windowTileHorizontal();
			} break;
			case WINDOWTILEVERTICAL: {
				windowTileVertical();
			} break;
			case WINDOWTILE: {
				windowTile();
			}
			case WINDOWLIST: {
				windowList();
			} break;
			case PROJECTNEW: {
				projectNew();
			} break;
			case PROJECTOPEN: {
				projectOpen();
			} break;
			case PROJECTSAVE: {
				projectSave();
			} break;
			case PROJECTCLOSE: {
				projectClose();
			} break;
		}
	}
	
    public void applicationExit() {
    	windowCloseAll();
    	projectClose();
    	if( m_mainFrame.getDesktop().getAllFrames().length == 0 && m_mainFrame.getProject() == null ) {
        	m_mainFrame.dispose();
    	}
    }

    public void fileClose() {
    	m_mainFrame.frameClose(m_mainFrame.getTopFrame());
    }

    public void fileGoTo() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }

    public void fileOpen() {
    	m_mainFrame.frameOpen();
    }

    public void filePrint() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }

    public void filePrintPageSetup() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }

    public void fileSave() {
    	m_mainFrame.frameSave(m_mainFrame.getTopFrame());
    }

    public void fileSaveAll() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
        for( int i=0; i<frames.length; i++ ) {
    		InternalTextFrame internalFrame = (InternalTextFrame)frames[i];
    		if( internalFrame.isUnnamed() ) {
    			fileSaveAs();
    		} else {
    			internalFrame.getDocument().saveFile(false);
    		}
        }
    }

    public void fileSaveAllExit() {
    	fileSaveAll();
    	applicationExit();
    }

    public void fileSaveAs() {
    	m_mainFrame.frameSaveAs(m_mainFrame.getTopFrame());
    }

    public void newItem() {
    	m_mainFrame.getDesktop().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        newText();
    	m_mainFrame.getDesktop().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

	int m_nUntitledCount = 0;
    public void newText() {
    	m_mainFrame.getDesktop().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        InternalTextFrame internalFrame = new InternalTextFrame("Text"+(++m_nUntitledCount));
        m_mainFrame.addInternalFrame(internalFrame);
    	m_mainFrame.getDesktop().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    public void workspaceClose() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }

    public void workspaceOpen() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }

    public void workspaceSave() {
        // This method is derived from interface gvid.command.FileCommands
        // to do: code goes here
    }
    
	public void windowCloseAll() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
        for( int i=0; i<frames.length; i++ ) {
    		try {
    			frames[i].setClosed(true);
    		} catch( PropertyVetoException e ) {
    			System.err.println(e);
    		}
        }
	}
	
	public void windowNext() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
		if( frames.length > 1 ) {
			frames[1].moveToFront();
    		try {
				frames[1].setSelected(true);
    		} catch( PropertyVetoException e ) {
    			System.err.println(e);
    		}
		}
	}
	
	public void windowPrev() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
		if( frames.length > 0 ) {
			frames[frames.length-1].moveToFront();
    		try {
				frames[frames.length-1].setSelected(true);
    		} catch( PropertyVetoException e ) {
    			System.err.println(e);
    		}
		}
	}
	
	public void windowCascade() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
        Point nextPos = new Point(0,0);
        int posIndex = 0;
        if( frames.length > 0 ) {
        	for( int i=frames.length-1; i >= 0; i-- ) {
    			try {
    				frames[i].setMaximum(false);
    				frames[i].setIcon(false);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
    			frames[i].setLocation(m_mainFrame.getCascadePosition(frames[i], posIndex++));
    			frames[i].setSize(m_mainFrame.getCascadeSize(desktop.getSize()));
    			Rectangle b = frames[i].getBounds();
    			if( ( b.x+b.width > desktop.getWidth() ) || 
    		    	( b.y+b.height > desktop.getHeight() ) ) {
    		    	posIndex = 0;
    				frames[i].setLocation(m_mainFrame.getCascadePosition(frames[i], posIndex++));
    			}
	//    		Point rootPane = frames[i].getRootPane().getLocation();
    			Icon icon = frames[i].getFrameIcon();
    			Insets border = frames[i].getBorder().getBorderInsets(frames[i]);
    			nextPos.x += icon.getIconWidth() + border.left + 2;
    			nextPos.y += icon.getIconHeight() + border.top + 2;
        	}
        	try {
        		frames[0].setSelected(true);
        	} catch( PropertyVetoException e ) {
        		System.err.println(e);
        	}
        }
	}
	
	public void windowTileHorizontal() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
        if( frames.length > 0 ) {
        	Dimension size = new Dimension(desktop.getSize().width, desktop.getSize().height/frames.length);
        	Point pos = new Point(0,0);
        	for( int i=0; i < frames.length; i++ ) {
    			try {
    				frames[i].setMaximum(false);
    				frames[i].setIcon(false);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
        		frames[i].setSize(size);
        		frames[i].setLocation(pos);
        		pos.y += size.height;
        	}
        }
	}
	
	public void windowTileVertical() {
        JDesktopPane desktop = m_mainFrame.getDesktop();
        JInternalFrame[] frames = desktop.getAllFrames();
        if( frames.length > 0 ) {
        	Dimension size = new Dimension(desktop.getSize().width/frames.length, desktop.getSize().height);
        	Point pos = new Point(0,0);
        	for( int i=0; i < frames.length; i++ ) {
    			try {
    				frames[i].setMaximum(false);
    				frames[i].setIcon(false);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
        		frames[i].setSize(size);
        		frames[i].setLocation(pos);
        		pos.x += size.width;
        	}
        }
	}
	
	public void windowTile() {
	}
	
	public void windowList() {
		WindowList.showDialog(m_mainFrame, m_mainFrame.getDesktop().getAllFrames());
		m_mainFrame.repaint();
	}
	
	public void newProject() {
		m_mainFrame.newProject();
		m_mainFrame.showProjectWindow(true);
	}
	
	public void windowMaximize() {
		InternalTextFrame topFrame = m_mainFrame.getTopFrame();
		if( topFrame != null ) {
    		try {
				topFrame.setMaximum(true);
    		} catch( java.beans.PropertyVetoException e ) {
        		System.err.println(e);
    		}        
		}
	}
	
	public void projectNew() {
		if( m_mainFrame.newProject() ) {
			m_mainFrame.showProjectWindow(true);
		}
	}
	
	public void projectOpen() {
		if( m_mainFrame.openProject() ) {
			m_mainFrame.showProjectWindow(true);
		}
	}
	
	public void projectSave() {
		m_mainFrame.saveProject();
	}
	
	public void projectClose() {
		if( m_mainFrame.closeProject() ) {
			m_mainFrame.showProjectWindow(false);
		}
	}
}