/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import gvid.ui.status.*;
import gvid.*;
import gvid.document.*;
import gvid.ui.splitscrollpane.*;
import gvid.ui.editor.*;
import gvid.ui.menu.*;
import gvid.command.*;
import gvid.project.*;

public class MainFrame 
	extends 	javax.swing.JFrame
	implements	DropTargetListener,
				VetoableChangeListener,
				ContainerListener,
				KeyListener,
				WindowListener,
				FocusListener {
	StatusPanel m_status = new StatusPanel();
	
	JDesktopPane m_documentPanel = new JDesktopPane();
	MainCommandHandler m_cmdHandler = new MainCommandHandler(this);
	JMenuBar m_menubar = new ViDMenuBar(m_cmdHandler, m_documentPanel);
	
	private static MainFrame m_mainFrame = null;
	public static MainFrame getMainFrame() {
		if( m_mainFrame == null ) {
			System.err.println("Building MainFrame");
			m_mainFrame = new MainFrame();
		}
		return m_mainFrame;
	}
	
	private MainFrame() {		
		Util.recurseCreateDropTarget(this, this, new Util.RecurseDropTargetComponentListener(this));
		
		m_documentPanel.addContainerListener(this);
		
		getContentPane().setLayout(new BorderLayout());
		setTitle("ViD");
		
		getContentPane().setLayout(new BorderLayout());
		
		getContentPane().add(m_status, BorderLayout.SOUTH);
		
		m_documentPanel.setOpaque(true);
		m_documentPanel.setBackground(Color.gray);
		m_documentPanel.setBorder(new gvid.ui.border.SimpleBevelBorder());
		getContentPane().add(m_documentPanel);
		
		setJMenuBar(m_menubar);
		
		Util.recurseAddKeyListener(this, this);
		
		javax.swing.FocusManager.setCurrentManager( new ViDFocusManager(this, m_documentPanel) );
		
		addWindowListener(this);
//		Util.returnMenuFocus(m_menubar);
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	}
	
	public InternalTextFrame getTopFrame() {
		InternalTextFrame rc = null;
        JInternalFrame[] frames = getDesktop().getAllFrames();
    	if( frames.length > 0 ) { 
    		rc = (InternalTextFrame)frames[0];
    	}
    	return rc;
	}
	
	public void setStatusSource( StatusSource src ) {
		m_status.setStatusSource(src);
	}
	
	public gvid.ui.status.StatusSource getStatusSource() {
		return m_status.getStatusSource();
	}
	
	public CommandHandler getCommandHandler() {
		return m_cmdHandler;
	}
	
	SplitScrollPane m_splitPane = null;
	SourceCodeView m_view = null;
	SourceCodeModel m_document = null;
	public void openFile( File f ) {
    	try {
    		// see if a frame matching this file is already open
    		JInternalFrame[] frames = m_documentPanel.getAllFrames();
        	InternalTextFrame internalFrame = null;
    		for( int j=0; j<frames.length; j++ ) {
    			InternalTextFrame textFrame = (InternalTextFrame)frames[j];
    			if( f.equals(textFrame.getDocument().getFile()) ) {
    				internalFrame = textFrame;
    			}
    		}
    		if( internalFrame == null ) {
    			// if no open frame was found, open a new one
        		internalFrame = new InternalTextFrame(f);
        		addInternalFrame( internalFrame );
    		} else {
    			// just move the currently open frame to the front
    			try {
    				internalFrame.setSelected(true);
    			} catch( PropertyVetoException e ) {
    				System.err.println(e);
    			}
    		}
    	} catch( FileNotFoundException e ) {
    		JOptionPane.showMessageDialog(m_mainFrame, "Cannot file file \""+f.getName()+"\"");
    	} catch( IOException e ) {
    		JOptionPane.showMessageDialog(m_mainFrame, "An error occurred while reading the file.");
    	}
/*		
		getContentPane();
		if( m_documentPanel != null ) {
			getContentPane().remove(m_documentPanel);
		}
		if( m_splitPane != null ) {
			m_documentPanel.remove(m_splitPane);
			m_splitPane = null;
		}
		if( m_document != null ) {
			m_document = null;
		}
		m_document = new SourceCodeModel();
		m_document.loadFile(f);
		m_view = new SourceCodeView();
		m_splitPane = new SplitScrollPane();
		m_view.setModel(m_document);
		m_splitPane.setView(m_view);
		getContentPane().add(m_splitPane, BorderLayout.CENTER);
		m_documentPanel = m_splitPane;
		getContentPane().validate();
*/		
	}
	
	MainFileCommandsImpl m_fileCommands = new MainFileCommandsImpl(this);
	public FileCommands getFileCommands() {
		return m_fileCommands;
	}
	
	MainViewCommandsImpl m_viewCommands = new MainViewCommandsImpl(this);
	public ViewCommands getViewCommands() {
		return m_viewCommands;
	}
	
	MainProjectCommandsImpl m_projectCommands = new MainProjectCommandsImpl(this);
	public ProjectCommands getProjectCommands() {
		return m_projectCommands;
	}
	
	public JDesktopPane getDesktop() {
		return m_documentPanel;
	}
	
	private WindowMenu m_windowMenu = null;
	
	public void frameOpen() {
    	// TODO : setup correct file dialog properties
    	m_fileDialog.setMultiSelectionEnabled(true);
    	m_fileDialog.setDialogTitle("Open File");
    	if( m_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION ) {
    		File[] files = m_fileDialog.getSelectedFiles();
    		if( files.length == 0 ) {
    			files = new File[1];
    			files[0] = m_fileDialog.getSelectedFile();
    		}
    		for( int i=0; i<files.length; i++ ) {
    			openFile( files[i] );
    		}
    	}
	}
	
	public boolean frameSave( InternalTextFrame frame ) {
		boolean rc = false;
		if( frame != null ) {
    		if( frame.isUnnamed() ) {
    			rc = frameSaveAs(frame);
    		} else {
    			rc = true;
    			frame.getDocument().saveFile(true);
    		}
		}
    	return rc;
	}
	
	JFileChooser m_fileDialog = new JFileChooser();
	public boolean frameSaveAs( InternalTextFrame frame ) {
		boolean rc = false;
		if( frame != null ) {
    		File saveFile = null;
    		// TODO : setup correct file dialog properties
    		m_fileDialog.setDialogTitle("Save File As");
    		while( ( saveFile == null ) && ( m_fileDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION ) ) {
    			saveFile = m_fileDialog.getSelectedFile();
    			try {
    				if( ( saveFile.exists() ) 
    				&& ( JOptionPane.showConfirmDialog(this, saveFile.getCanonicalPath()+" exists. Overwrite?", "Overwrite Warning", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION ) ) {
    					saveFile = null;
    				} else {
    					frame.setFile(saveFile);
    					rc = frameSave(frame);
    				}
    			} catch( IOException e ) {
    				saveFile = null;
    			}
    		}
		}
    	return rc;
	}
	
	public void frameClose( InternalTextFrame frame ) {
		if( frame != null ) {
    		try {
    			frame.setClosed(true);
    		} catch( PropertyVetoException e ) {
    			System.err.println(e);
    		}
		}
	}
	
    public int m_newFramePos = 0;
	public void addInternalFrame( JInternalFrame internalFrame ) {
		// TODO : polish adding new windows
        JDesktopPane desktop = m_mainFrame.getDesktop();
    	internalFrame.setLocation(getCascadePosition(internalFrame, m_newFramePos));
    	Dimension d = getCascadeSize(desktop.getSize());
    	if( internalFrame.getLocation().x+d.width > desktop.getSize().width ) {
    		m_newFramePos = 0;
    		internalFrame.setLocation(getCascadePosition(internalFrame, m_newFramePos));
    	}
    	d.height = desktop.getHeight()-internalFrame.getLocation().y;
    	internalFrame.setSize(d);
		m_newFramePos++;
    	internalFrame.setVisible(true);
    	desktop.add(internalFrame);
    	try {
        	internalFrame.setSelected(true);
    	} catch( java.beans.PropertyVetoException e ) {
        	System.err.println(e);
    	}        
	}
	
	public Point getCascadePosition( JInternalFrame frame, int pos ) {
    	Icon icon = frame.getFrameIcon();
    	Insets border = frame.getBorder().getBorderInsets(frame);
    	return new Point((icon.getIconWidth()+border.left+2)*pos, (icon.getIconHeight()+border.top+2)*pos);
	}
	
	public Dimension getCascadeSize( Dimension parentSize ) {
		return new Dimension((int)(parentSize.width*0.80), (int)(parentSize.height*0.70));
	}
///////////////////////////////////////////////////////////////////////////////
//	DropTargetListener	
	DropTarget m_dropTarget = new DropTarget(this, this);

	public void dragEnter(DropTargetDragEvent dtde) {
	}
	
	public void dragOver(DropTargetDragEvent dtde) {
		if( !dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor) ) {
			dtde.rejectDrag();
		}
	}
	
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}
	
	public void dragExit(DropTargetEvent dte) {
	}
	
	public void drop(DropTargetDropEvent dtde) {
//		System.err.println("drop: "+dtde);
		
//		DropTargetContext context = dtde.getDropTargetContext();
//		System.err.println("context: "+context);
		
		Transferable trans = dtde.getTransferable();
//		System.err.println("transferrable: "+trans);
		
		if( dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor) ) {
			try {
				dtde.acceptDrop(DnDConstants.ACTION_COPY);
				java.util.List files = (java.util.List)trans.getTransferData(DataFlavor.javaFileListFlavor);
				Iterator iter = files.iterator();
				while( iter.hasNext() ) {
					File f = (File)iter.next();
					if( f.getName().endsWith(".vpf") ) {
    					openProject(f);
					} else {
						openFile(f);
					}
//					System.err.println("Files dropped: "+f);
				}
				dtde.dropComplete(true);
			} catch( IOException e ) {
				System.err.println(e);
			} catch( UnsupportedFlavorException e ) {
				System.err.println(e);
			}
		}
	}
//	DropTargetListener	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	VetoableChangeListener	
	public void vetoableChange(PropertyChangeEvent evt) 
	throws PropertyVetoException {
		if( evt.getSource() instanceof InternalTextFrame ) {
			InternalTextFrame frame = (InternalTextFrame)evt.getSource();
			if( evt.getPropertyName().equals( JInternalFrame.IS_CLOSED_PROPERTY) && evt.getNewValue().equals(Boolean.TRUE) ) {
				if( frame.getDocument().isDirty() ) {
					int option = JOptionPane.showConfirmDialog(this, frame.getTitle()+" has changed. Would you like to save it first?");
					if( option == JOptionPane.YES_OPTION ) {
						if( !frameSave(frame) ) {
							throw new PropertyVetoException("Frame not saved. Close operation cancelled.", evt);
						}
					} else if( option == JOptionPane.NO_OPTION ) {
					} else {
						throw new PropertyVetoException("Frame not saved. Close operation cancelled.", evt);
					}
				}
			}
		}
	}
//	VetoableChangeListener	
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ContainerListener
    public void componentAdded(ContainerEvent e) { 
    	if( e.getChild() instanceof JComponent ) {
    		((JComponent)e.getChild()).addVetoableChangeListener(this);
    	}
    	if( m_documentPanel.getComponentCount() > 0 ) {
    		InternalTextFrame itf = (InternalTextFrame)m_documentPanel.getComponent(0);
    		setStatusSource(itf.getStatusSource());
    	} else {
    		setStatusSource( null );
    	}
    }
    
    public void componentRemoved(ContainerEvent e) { 
    	if( e.getChild() instanceof JComponent ) {
    		((JComponent)e.getChild()).removeVetoableChangeListener(this);
    	}
    	if( m_documentPanel.getComponentCount() > 0 ) {
    		InternalTextFrame itf = (InternalTextFrame)m_documentPanel.getComponent(0);
    		setStatusSource(itf.getStatusSource());
    	} else {
    		setStatusSource( null );
    	}
    }    
//	ContainerListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	KeyListener
	public void keyTyped(KeyEvent e) {
	}

	private static final int MODMASK = InputEvent.CTRL_MASK|InputEvent.ALT_MASK|InputEvent.SHIFT_MASK;
	public void keyPressed(KeyEvent e) {
		CommandKey key = new CommandKey(e.getKeyCode(), e.getModifiers()&MODMASK);
		CommandId cmdId = (CommandId)m_cmdHandler.getCommandKeyMap().get(key);
		if( cmdId != null ) {
			m_cmdHandler.executeCommand(cmdId.getGroup(), cmdId.getCmd());
			e.consume();
		}
	}

	public void keyReleased(KeyEvent e) {
	}
//	KeyListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	WindowListener
    public void windowOpened(WindowEvent e) { }
    public void windowClosing(WindowEvent e) { 
    	getFileCommands().applicationExit(); 
    }
    public void windowClosed(WindowEvent e) { 
    	System.exit(0);
    }
    public void windowIconified(WindowEvent e) { }
    public void windowDeiconified(WindowEvent e) { }
    public void windowActivated(WindowEvent e) { }
    public void windowDeactivated(WindowEvent e) { }
//	WindowListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	FocusListener
	public void focusGained(FocusEvent e) {
		System.err.println("Gained "+e.getSource() );
/*
    	javax.swing.JDesktopPane desktop = m_mainFrame.getDesktop();
    	if( desktop.getComponentCount() > 0 ) {
    		java.awt.Component c = desktop.getComponent(0);
    		c.requestFocus();
    	}
*/    	
	}

	public void focusLost(FocusEvent e) {
		System.err.println("Lost "+e.getSource() );
	}
//	FocusListener
///////////////////////////////////////////////////////////////////////////////
	public static void main( String[] args ) {
		try {
			  UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			  com.sun.java.swing.plaf.windows.WindowsLookAndFeel laf = new com.sun.java.swing.plaf.windows.WindowsLookAndFeel();
//			LookAndFeel laf = new com.sun.java.swing.plaf.windows.WindowsLookAndFeel() {
//				public boolean isNativeLookAndFeel() {
//					return true;
//				}
//			};
			UIManager.setLookAndFeel(laf);
		} catch( Exception e ) {
			System.err.println(e);
		}
//		String[] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
//		for( int i=0; i<fonts.length; i++ ) {
//			System.err.println(fonts[i]);
//		}
		MainFrame f = MainFrame.getMainFrame();
		f.setStatusSource(null);
/*
		f.setStatusSource(new StatusSource() {
			String[] m_str = {
				"Ln 1, Col 1   ",
				"REC",
				"COL",
				"OVR",
				"READ"
			};
			boolean [] m_active = {
				true,
				false,
				false,
				false,
				false
			};
			public void addStatusListener( StatusListener l ) {
			}
			public void removeStatusListener( StatusListener l ) {
			}
			
			public String getStatusMsg() {
				return "Ready";
			}
			
			public void setStatusMsg( String str ) {}
			
			public int getStatusCount() {
				return 5;
			}
			public String getStatusString(int index) {
				return m_str[index];
			}
			public boolean isStatusActive(int index) {
				return m_active[index];
			}
		});
*/		
		f.setSize(300,500);
		f.setVisible(true);
	}
	
	ProjectWindow m_projectWindow = new ProjectWindow();
	
	public ProjectModel getProject() {
		ProjectModel rc = null;
		if( m_projectWindow != null ) {
			rc = m_projectWindow.getProject();
		}
		return rc;
	}
	
	public boolean newProject() {
		boolean rc = false;
		if( m_projectWindow.conditionalSaveProject() ) {
    		File projectFile = null;
	    	m_fileDialog.setDialogTitle("Save New Project As");
    		while( ( projectFile == null ) && ( m_fileDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION ) ) {
    			projectFile = m_fileDialog.getSelectedFile();
    			try {
    				if( ( !projectFile.exists() ) 
    				|| ( JOptionPane.showConfirmDialog(this, projectFile.getCanonicalPath()+" exists. Overwrite?", "Overwrite Warning", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)==JOptionPane.YES_OPTION ) ) {
						rc = m_projectWindow.newProject(projectFile);
						rc = true;
    				}
    			} catch( IOException e ) {
    				System.err.println(e);
    			}
    		}
		}
		m_cmdHandler.isGroupAvailable(CommandBase.PROJECTGROUP);
		return rc;
	}
	
	public boolean saveProject() {
		return m_projectWindow.saveProject();
	}
	
	public boolean openProject( File file ) {
		boolean rc = m_projectWindow.openProject(file);
		m_cmdHandler.isGroupAvailable(CommandBase.PROJECTGROUP);
		return rc;
	}
	
	public boolean openProject() {
		boolean rc = false;
    	// TODO : setup correct file dialog properties
    	m_fileDialog.setMultiSelectionEnabled(false);
    	m_fileDialog.setDialogTitle("Open Project");
    	if( m_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION ) {
    		File file = m_fileDialog.getSelectedFile();
    		rc = openProject(file);
    	}
    	return rc;
	}
	
	public boolean closeProject() {
		boolean rc = m_projectWindow.closeProject();
		m_cmdHandler.isGroupAvailable(CommandBase.PROJECTGROUP);
		return rc;
	}
	
	public void showProjectWindow(boolean bVisible) {
		m_projectWindow.setVisible(bVisible);
	}
	
	public boolean isProjectWindowShowing() {
		return m_projectWindow.isVisible();
	}
}