/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
public class ViDFocusManager extends javax.swing.FocusManager {
	private javax.swing.FocusManager m_defaultManager = new javax.swing.DefaultFocusManager();
	private Component m_mdiRoot = null;
	private JDesktopPane m_desktop = null;
	private int m_index = 0;
	private JInternalFrame[] m_frames = null;
	
	public ViDFocusManager( Component mdiRoot, JDesktopPane desktop ) {
		m_mdiRoot = mdiRoot;
		m_desktop = desktop;
	}
	
    public void focusNextComponent(Component aComponent) {
    	if( SwingUtilities.getRoot(aComponent) != m_mdiRoot ) {
    		m_defaultManager.focusNextComponent(aComponent);
    	} else {
    		if( m_frames == null ) {
    			m_frames = m_desktop.getAllFrames();
    			m_index = 0;
    		}
    		// restore the original order
    		if( m_index != 0 ) {
    			m_desktop.remove(m_frames[m_index]);
    			m_desktop.add(m_frames[m_index], m_index);
    		}
    		// update the index
    		m_index = (m_index+1)%m_frames.length;
//    		System.err.println("Next Index: "+m_index);
    		// set the new order
    		if( m_index != 0 ) {
    			m_desktop.remove(m_frames[m_index]);
    			m_desktop.add(m_frames[m_index], 0);
    		}
    		// select the top frame
    		try {
    			m_frames[m_index].setSelected(true);
    		} catch( java.beans.PropertyVetoException e ) {
    			System.err.println(e);
    		}
    	}
    }

    public void focusPreviousComponent(Component aComponent) {
    	if( SwingUtilities.getRoot(aComponent) != m_mdiRoot ) {
    		m_defaultManager.focusPreviousComponent(aComponent);
    	} else {
    		if( m_frames == null ) {
    			m_frames = m_desktop.getAllFrames();
    			m_index = 0;
    		}
    		// restore the original order
    		if( m_index != 0 ) {
    			m_desktop.remove(m_frames[m_index]);
    			m_desktop.add(m_frames[m_index], m_index);
    		}
    		// update the index
    		m_index = (m_index+m_frames.length-1)%m_frames.length;    		
    		System.err.println("Previous Index: "+m_index);
    		// set the new order
    		if( m_index != 0 ) {
    			m_desktop.remove(m_frames[m_index]);
    			m_desktop.add(m_frames[m_index], 0);
    		}
    		// select the top frame
    		try {
    			m_frames[m_index].setSelected(true);
    		} catch( java.beans.PropertyVetoException e ) {
    			System.err.println(e);
    		}
    	}
    }

    public void processKeyEvent(Component focusedComponent, KeyEvent anEvent) {
    	JComponent jcomp = null;
    	if( focusedComponent instanceof JComponent ) {
    		jcomp = (JComponent)focusedComponent;
    	}
    	if( SwingUtilities.getRoot(focusedComponent) != m_mdiRoot ) {
    		m_defaultManager.processKeyEvent(focusedComponent, anEvent);
    	} else {
    		if( anEvent.getID() == KeyEvent.KEY_PRESSED ) {
    			if( anEvent.getKeyCode() == KeyEvent.VK_CONTROL ) {
    				m_frames = m_desktop.getAllFrames();
    				m_index = 0;
    				anEvent.consume();
    			} else if( anEvent.getKeyCode() == KeyEvent.VK_TAB ) {
    				if( jcomp==null || !jcomp.isManagingFocus() ) {
    					if( (anEvent.getModifiers()&InputEvent.SHIFT_MASK) != 0 ) {
    						focusPreviousComponent( focusedComponent );
    					} else {
    						focusNextComponent( focusedComponent );
    					}
    					anEvent.consume();
    				}
    			} else {
    				// reset the order
    				m_frames = null;
    			}
    		} else if( anEvent.getID() == KeyEvent.KEY_RELEASED ) {
    			if( anEvent.getKeyCode() == KeyEvent.VK_CONTROL ) {
    				m_frames = null;
    				anEvent.consume();
    			}
    		}
    	}
    }
}