/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor;

import java.awt.event.*;
import java.awt.*;
import javax.swing.event.*;
import javax.swing.*;
import gvid.document.*;
import gvid.util.*;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.dnd.*;

public class EditCaret 
    extends     javax.swing.JComponent
    implements  ActionListener,
                ComponentListener,
                ReferencePointListener,
                HierarchyListener,
                Autoscroll {
    public static final int INSERTMODE = 0;
    public static final int OVERWRITEMODE = 1;

    private static int m_mode = INSERTMODE;
    private static Vector m_instances = new Vector();
    
    private Point m_position = new Point();
    private SourceCodeEditor m_editor = null;
    private Timer m_blinkTimer = new Timer(467, this);
    private boolean m_bClearXOR = false;
    private boolean m_bMovingMode = false;
	private String m_pointId = "";
    
	public EditCaret( SourceCodeEditor editor ) {
	    setView(editor);
		setCursor( Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
	    addComponentListener(this);
	    addHierarchyListener(this);
	}
	
	Vector m_caretListeners = new Vector();
	public void addCaretListener( EditCaretListener l ) {
		m_caretListeners.addElement(l);
	}
	
	public void removeCaretListener( EditCaretListener l ) {
		m_caretListeners.removeElement(l);
	}
	
	private void notifyCaretListeners(EditCaretEvent evt) {
		Enumeration enum = m_caretListeners.elements();
		while( enum.hasMoreElements() ) {
			EditCaretListener l = (EditCaretListener)enum.nextElement();
			l.caretUpdate(evt);
		}
	}
	
	public void copySettingsFrom( EditCaret caret ) {
		if( caret != this ){
	    	m_position = new Point(caret.m_position);
	    	m_editor.setReferencePoint(m_editor.posToDoc(m_position), m_pointId, this);
		}
	}
	
	public void setCaretPosition(int x, int y) {
		if( m_editor.getSourceModel() != null ) {
	    	if( x < m_editor.getSourceModel().getColumnMinimum() ) {
	        	x = m_editor.getSourceModel().getColumnMinimum();
	    	}
	    	if( y < m_editor.getFirstLine() ) {
	        	y = m_editor.getFirstLine();
	    	}
	    	if( x > m_editor.getSourceModel().getColumnMaximum() ) {
	        	x = m_editor.getSourceModel().getColumnMaximum();
	    	}
	    	if( y >= m_editor.getLineCount() ) {
	        	y = m_editor.getLineCount();
	    	}
		} else {
			x = 0;
			y = 0;
		}
	    if( m_position.x != x || m_position.y != y ) {
	        m_position.x = x;
	        m_position.y = y;
	        if( m_pointId.length() != 0 ) {
		    	m_editor.setReferencePoint(m_editor.posToDoc(m_position), m_pointId, this);	        
	        }
	        updateBounds();
	        notifyCaretListeners( new EditCaretEvent(
	        	this, 
	        	EditCaretEvent.POSITIONCHANGE, 
	        	getCaretPosition(),
	        	m_mode) );
	    }
	}
	
	public void setCaretPosition(Point p) {
	    setCaretPosition(p.x, p.y);
	}
	
	public Point getCaretPosition() {
	    return new Point(m_position);
	}
	
	public void setMovingMode( boolean bMoving ) {
		m_bMovingMode = bMoving;
		updateBounds();
	}
	
	public static void setInsert() {
		if( m_mode != INSERTMODE ) {
	    	m_mode = INSERTMODE;
	    	updateInstances();
		}
	}
	
	public static void setOverwrite() {
		if( m_mode != OVERWRITEMODE ) {
	    	m_mode = OVERWRITEMODE;
	    	updateInstances();
	    }
	}
	
	public static boolean isInsert() {
	    return (m_mode == INSERTMODE);
	}
	
	public static boolean isOverwrite() {
	    return (m_mode == OVERWRITEMODE);
	}
	
	public void setView( SourceCodeEditor editor ) {
		if( m_editor != null ) {
			m_editor.removeReferencePoint(m_pointId, this);
		}
	    m_editor = editor;
	    setCaretPosition(0,0);
	}
	
    public void updateBounds() {
//    	System.err.println("editCaret: updateBounds");
        clearXOR();
        Point  p = getPreferredLocation();
        Dimension d = getPreferredSize();
        Rectangle bounds = new Rectangle(p.x, p.y, d.width, d.height);
	    m_blinkTimer.restart();        
	    m_bClearXOR = false;
        if( getParent() != null ) {
            getParent().invalidate();
        }
        Rectangle intersect = m_editor.getEditBounds().intersection(bounds);
        if( intersect.width <= 0 || intersect.height <= 0 ) {
        	boolean temp = m_bExternallyVisible;
        	setVisible(false);
        	m_bExternallyVisible = temp;
        } else {
        	bounds.x += Math.abs(bounds.width-intersect.width);
        	setVisible(m_bExternallyVisible);
        }
        setBounds(bounds);
    }
    
    // this is used to remember if the caret should be visible based on
    // external control.
    private boolean m_bExternallyVisible = true;
    public void setVisible( boolean v ) {
    	super.setVisible(v);
    	m_bExternallyVisible = v;
    }
    
    public Point getPreferredLocation() {
        Point rc = new Point();
        Point adjusted = m_editor.docToPos(m_editor.posToDoc(m_position));
        if( m_editor != null ) {
        	if( isInsert() ) {
            	rc.x = m_editor.getColumnLeftEdge(adjusted.x)-1;
        	} else {
            	rc.x = m_editor.getColumnLeftEdge(adjusted.x);
        	}
            rc.y = m_editor.getRowTopEdge(adjusted.y);
        }
//        System.err.println("("+rc.x+", "+rc.y+")");
        return rc;
    }

    public void paint(Graphics g) {
        super.paint(g);
        if( m_bClearXOR ) {
        } else {
            g.setColor(Color.black);
            g.setXORMode(Color.white);
            if( !m_bMovingMode ) {
            	g.fillRect(0, 0, getBounds().width, getBounds().height);
            } else {
            	ditherArea(new Rectangle(0, 0, getBounds().width, getBounds().height), g);
            }
        }
        m_bClearXOR = !m_bClearXOR;
    }
    
    public void clearXOR() {
        if( m_bClearXOR ) {
            paintImmediately(0, 0, getSize().width, getSize().height);
        }
    }
    
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }
    
    public Dimension getPreferredSize() {
        Dimension rc = new Dimension();
        if( m_editor != null ) {
        	if( m_mode==INSERTMODE || m_bMovingMode ) {
                rc.width = 2;
                rc.height = m_editor.getFontHeight();
        	} else {
                rc.width = m_editor.getFontWidth();
                rc.height = m_editor.getFontHeight();                    
        	}
        }
        return rc;
    }
    
///////////////////////////////////////////////////////////////////////////////
// ActionListener
    public void actionPerformed( ActionEvent e ) {
        repaint();
    }
// ActionListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ComponentListener
    public void componentResized(ComponentEvent e) {
    }
    public void componentMoved(ComponentEvent e) {
    }
    public void componentShown(ComponentEvent e) {
	    m_blinkTimer.restart();
	    m_bClearXOR = false;
    }
    public void componentHidden(ComponentEvent e) {
        m_blinkTimer.stop();
        m_bClearXOR = false;
    }
// ComponentListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
// ReferencePointListener
	public void pointChanged( Point p, String name ) {
		if( name.equals(m_pointId ) ) {
			Point currentPos = m_editor.posToDoc(m_position);
			if( !currentPos.equals(p) ) {
				Point newPos = m_editor.docToPos(p);
				setCaretPosition(newPos);
			}
		}
	}
	
	public String changeReferenceName( String name ) {
		if( name.equals(m_pointId) ) {
	    	m_pointId = "CARET_"+m_editor.getUnusedReferenceId();
		}
		return m_pointId;
	}
	
// ReferencePointListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	HierarchyListener
	public void hierarchyChanged(HierarchyEvent e) {
		if( (e.getChangeFlags()&HierarchyEvent.PARENT_CHANGED) != 0 ) {
			if( e.getChanged() == this ) {
				if( getParent() != null ) {
	    			m_instances.addElement(this);
	    			updateBounds();
	    			m_blinkTimer.restart();
	    			m_bClearXOR = false;
	    			if( m_editor != null ) {
	    				m_pointId = "CARET_"+m_editor.getUnusedReferenceId();
	    				m_editor.addReferencePoint(m_position, m_pointId, this);
	    			}
				} else {
	    			m_blinkTimer.stop();
    				m_bClearXOR = false;
	    			m_instances.removeElement(this);
					m_editor.removeReferencePoint(m_pointId, this);
					m_pointId = "";
				}
//				System.err.println(e);
			}
		}
	}
//	HierarchyListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	Autoscroll
	public Insets getAutoscrollInsets() {
		return new Insets(getHeight()/2, getWidth()/2, getHeight()/2, getWidth()/2);
	}
	
	public void autoscroll(Point cursorLocn) {
		if( ( getParent() != null ) && ( getParent() instanceof Autoscroll ) ) {
			cursorLocn = SwingUtilities.convertPoint(this, cursorLocn, getParent());
			Rectangle b = getParent().getBounds();
			Insets asi = ((Autoscroll)getParent()).getAutoscrollInsets();
			if( ( cursorLocn.y < asi.top ) || 
				( cursorLocn.y > b.height-asi.bottom ) ||
				( cursorLocn.x < asi.left ) ||
				( cursorLocn.x > b.width-asi.right ) ) {
//				System.err.println("Autoscrolling parent");
				((Autoscroll)getParent()).autoscroll(cursorLocn);
			}
		}
	}
//	Autoscroll
///////////////////////////////////////////////////////////////////////////////
	
	private Image m_ditherImage = null;
	private Rectangle m_lastImageSize = null;
	private void ditherArea( Rectangle b, Graphics g ) {
	    if( m_ditherImage == null || !m_lastImageSize.equals(b) ) {
	        m_lastImageSize = new Rectangle(b);
	        m_ditherImage = createImage(b.width, b.height);
	        Graphics g2 = m_ditherImage.getGraphics();
	        g2.setColor(Color.white);
	        g2.fillRect(0, 0, b.width, b.height);
	        g2.setColor(Color.black);
	        for( int y=b.y-b.width-1; y<b.y+b.height+1; y+=2 ) {
	            g2.drawLine(0,y,b.width+1,y+b.width+1);
	        }
	    }
	    
	    g.drawImage(m_ditherImage, b.x, b.y, null);
	}
	
	// mode changed are global across carets. This is called when it changes
	private static void updateInstances() {
		Enumeration enum = m_instances.elements();
		while( enum.hasMoreElements() ) {
			EditCaret caret = (EditCaret)enum.nextElement();
			caret.updateBounds();
			caret.notifyCaretListeners( 
				new EditCaretEvent(
					caret, 
					EditCaretEvent.MODECHANGE, 
					caret.getCaretPosition(),
					EditCaret.m_mode)	);			
		}
	}
}