/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.selection;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import gvid.ui.editor.SourceCodeEditor;
import java.awt.datatransfer.*;
import java.io.*;
import java.awt.dnd.*;
import gvid.util.ReferencePointListener;
import gvid.undo.*;
import gvid.ui.editor.undo.*;

public class LineSelection 
	extends javax.swing.JComponent 
	implements	SourceCodeSelection,
				ReferencePointListener,
				HierarchyListener,
				Autoscroll {

	SourceCodeEditor m_editor;
	Point m_posStart = new Point();
	Point m_posEnd = new Point();
	int m_lineStart = 0;
	int m_lineEnd = 0;
	private String m_copyData = "";
	private DragSource m_dragSource = null;
	private DragGestureRecognizer m_dragGestureRecognizer = null;
	private String m_startId = "";
	private String m_endId = "";
	
	public LineSelection( Point start, Point end, SourceCodeEditor editor ) {
		m_editor = editor;
		
		m_posStart = editor.docToPos(editor.posToDoc(start));
		m_posEnd = editor.docToPos(editor.posToDoc(end));
		m_lineStart = m_posStart.y;
		m_lineEnd = m_posEnd.y;
		adjustPositions();
		
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));		
		
		m_dragSource = new DragSource();
		m_dragGestureRecognizer = m_dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY_OR_MOVE, this);
		
		m_startId = "LINE_START_"+editor.getUnusedReferenceId();
		m_endId = "LINE_END_"+editor.getUnusedReferenceId();
		editor.setReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
		editor.setReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
		
		addHierarchyListener(this);
	}
	
	private void adjustPositions() {
		if( m_posStart.y == m_posEnd.y ) {
			if( m_posStart.x > m_posEnd.x ) {
				m_posEnd.x = 0;
				m_posStart.x = 0;
				m_posStart.y = m_lineStart+1;
			} else {
				m_posStart.x = 0;
				m_posEnd.x = 0;
				m_posEnd.y = m_lineEnd+1;
			}
		} else if( m_posStart.y < m_posEnd.y ) {
			m_posStart.x = 0;
			m_posEnd.x = 0;
			m_posEnd.y = m_lineEnd+1;
		} else if( m_posStart.y > m_posEnd.y ) {
			m_posEnd.x = 0;
			m_posStart.x = 0;
			m_posStart.y = m_lineStart+1;
		}
	}
	
	public LineSelection( Point start, Point end, String str ) {
		m_posStart = new Point(start);
		m_posEnd = new Point(end);
		m_lineStart = m_posStart.y;
		m_lineEnd = m_posEnd.y;
		adjustPositions();
		m_copyData = str;
	}
	
 	public boolean isFocusTraversable() {
		return false;
	}
	
   public Component asComponent() {
    	return this;
    }
    
	public SourceCodeSelection createCopy(SourceCodeEditor editor) {
		return new LineSelection(m_posStart, m_posEnd, editor);
	}
	
	public int getSelectionType() {
		return LINE;
	}

	public void setSelectionStart(Point selStart) {
    	SourceCodeEditor editor = m_editor;
		m_posStart = editor.docToPos(editor.posToDoc(selStart));
		m_lineStart = m_posStart.y;
		adjustPositions();
		updateBounds();
		editor.setReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
		validateSelection();
	}
	
	public void setSelectionEnd(Point selEnd) {
    	SourceCodeEditor editor = m_editor;
		m_posEnd = editor.docToPos(editor.posToDoc(selEnd));
		m_lineEnd = m_posEnd.y;
		adjustPositions();
		updateBounds();
		editor.setReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
		validateSelection();
	}
	
	public Point getSelectionEnd() {
		return m_posEnd;
	}

	public java.awt.Point getSelectionStart() {
		return m_posStart;
	}
		
	public void paint( Graphics g ) {
		Point pl = getLocation();
    	SourceCodeEditor editor = m_editor;
		
		Point front = null;
		Point back = null;
		
		if( m_posStart.y < m_posEnd.y ) {
			front = m_posStart;
			back = m_posEnd;
		} else if( m_posStart.y > m_posEnd.y ) {
			front = m_posEnd;
			back = m_posStart;
		} else {
			if( m_posStart.x < m_posEnd.x ) {
				front = m_posStart;
				back = m_posEnd;
			} else if( m_posStart.x > m_posEnd.x ) {
				front = m_posEnd;
				back = m_posStart;
			} else {
				return;
			}
		}
		
		g.setColor(Color.white);
		g.setXORMode(Color.black);
		
		// draw middle lines
		for( int i = front.y; i < back.y; i++ ) {
			Point lineEnd = editor.positionToView(editor.docToPos(new Point(editor.getSourceModel().lineLength(i), i)));
			g.fillRect(editor.getEditBounds().x-pl.x, lineEnd.y-pl.y, lineEnd.x-editor.getEditBounds().x+editor.getFontWidth(), editor.getFontHeight());
		}
		
		g.setPaintMode();
	}
	
    public void updateBounds() {
        Point  p = getPreferredLocation();
        Dimension d = getPreferredSize();
        Rectangle bounds = new Rectangle(p.x, p.y, d.width, d.height);
        setBounds(bounds);
        if( getParent() != null ) {
            getParent().invalidate();
        }
    }
    
	public void swapAnchor() {
		Point temp = m_posStart;
    	SourceCodeEditor editor = m_editor;
		m_posStart = m_posEnd;
		m_posEnd = temp;
		editor.setReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
		editor.setReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
	}
	
    public Point getPreferredLocation() {
    	Point rc = new Point();
    	SourceCodeEditor editor = m_editor;
    	
    	// Find the corner of the upper left selection
		if( m_posStart.y < m_posEnd.y ) {
			rc = editor.positionToView(m_posStart);
		} else if( m_posStart.y > m_posEnd.y ) {
			rc = editor.positionToView(m_posEnd);
		} else {
			if( m_posStart.x < m_posEnd.x ) {
				rc = editor.positionToView(m_posStart);
			} else {
				rc = editor.positionToView(m_posEnd);
			}
		}
		
		// if both positions are not on the same line, the left side is the edge of the edit area
		if( m_posStart.y != m_posEnd.y ) {
			rc.x = editor.getEditBounds().x;
		} else {
			rc.x = Math.max( rc.x, editor.getEditBounds().x );
		}
    	
        return rc;
    }

    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    
    public Dimension getMaximumSize() {
        return getPreferredSize();
    }
    
    public Dimension getPreferredSize() {
        Dimension rc = new Dimension();
    	SourceCodeEditor editor = m_editor;
        
		rc.height = (Math.abs(m_posStart.y-m_posEnd.y)+1)*editor.getFontHeight();
		if( m_posStart.y != m_posEnd.y ) {
			rc.width = editor.getEditBounds().width;
		} else {
			int viewStart = editor.positionToView(m_posStart).x;
			int viewEnd = editor.positionToView(m_posEnd).x;
			int editLeft = editor.getEditBounds().x;
			rc.width = Math.abs( Math.max(viewStart, editLeft) - Math.max(viewEnd, editLeft) );
		}
		
        return rc;
    }
    
    public boolean containsPos( Point pos ) {
    	boolean rc = true;
    	
    	if( pos.y < Math.min(m_posStart.y, m_posEnd.y) ) {
    		rc = false;
    	}
    	
    	if( pos.y == m_posStart.y ) {
    		if( m_posStart.y < m_posEnd.y && pos.x <= m_posStart.x ) {
    			rc = false;    			
    		} else if( m_posStart.y > m_posEnd.y && pos.x >= m_posStart.x ) {
    			rc = false;
    		} else if( m_posStart.y == m_posEnd.y && (
    		           pos.x <= Math.min(m_posStart.x, m_posEnd.x) || 
    		           pos.x >= Math.max(m_posStart.x, m_posEnd.x) ) ) {
    		    rc = false;
    		}
    	}
    	
    	if( pos.y == m_posEnd.y ) {
    		if( m_posEnd.y < m_posStart.y && pos.x <= m_posEnd.x ) {
    			rc = false;    			
    		} else if( m_posEnd.y > m_posStart.y && pos.x >= m_posEnd.x ) {
    			rc = false;
    		}
    	}
    	
    	if( pos.y > Math.max(m_posStart.y, m_posEnd.y) ) {
    		rc = false;
    	}
    	
    	return rc;
    }
    
    public boolean contains( Point p ) {
    	return contains(p.x, p.y);
    }
    
    public boolean contains( int x, int y ) {
    	boolean rc = false;
    	SourceCodeEditor editor = m_editor;
		Point pl = getLocation();
		
		Point front = null;
		Point back = null;
		
		if( m_posStart.y < m_posEnd.y ) {
			front = m_posStart;
			back = m_posEnd;
		} else if( m_posStart.y > m_posEnd.y ) {
			front = m_posEnd;
			back = m_posStart;
		} else {
			if( m_posStart.x < m_posEnd.x ) {
				front = m_posStart;
				back = m_posEnd;
			} else if( m_posStart.x > m_posEnd.x ) {
				front = m_posEnd;
				back = m_posStart;
			} else {
				return false;
			}
		}
		
		if( front.y == back.y ) {
			// selection is on a single line
			Point lineStart = editor.positionToView(front);
			Point lineEnd = editor.positionToView(back);
			Rectangle r = new Rectangle(lineStart.x-pl.x, lineStart.y-pl.y, lineEnd.x-lineStart.x, editor.getFontHeight());
			if( r.contains(x,y) ) {
				rc = true; 
			}
		} else {
			// selection is on multiple lines			
			// draw the first line
			Point lineStart = editor.positionToView(front);
			Point lineEnd = editor.positionToView(new Point(editor.getSourceModel().lineLength(front.y), front.y));
			Rectangle r = new Rectangle(lineStart.x-pl.x, lineStart.y-pl.y, editor.getEditBounds().x+editor.getEditBounds().width-lineStart.x, editor.getFontHeight());
			if( r.contains(x,y) ) {
				rc = true; 
			}

			// draw middle lines
			for( int i = front.y+1; i < back.y; i++ ) {
				lineEnd = editor.positionToView(editor.docToPos(new Point(editor.getSourceModel().lineLength(i), i)));
				r = new Rectangle(editor.getEditBounds().x-pl.x, lineEnd.y-pl.y, editor.getEditBounds().x+editor.getEditBounds().width, editor.getFontHeight());
				if( r.contains(x,y) ) {
					rc = true; 
					break;
				}
			}

			// draw the last line
			lineEnd = editor.positionToView(back);
			r = new Rectangle(editor.getEditBounds().x-pl.x, lineEnd.y-pl.y, lineEnd.x-editor.getEditBounds().x, editor.getFontHeight());			
			if( r.contains(x,y) ) {
				rc = true; 
			}
		}
    	return rc;
    }
    private void setAppropriateCursor(Point p) {
		if( contains(p) ) {
			setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		} else {
			setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
		}
    }

    public UndoCommand paste( SourceCodeEditor editor, Point dest ) {
		Point docDest = editor.posToDoc(dest);
		m_copyData = getSelectedText();
    	UndoCommand rc = editor.getSourceModel().insertText(m_copyData, docDest);
    	return rc;
    }
    
    public UndoCommand delete() {
    	UndoChain rc = new UndoChain("selectionDelete");
    	SourceCodeEditor editor = m_editor;
    	Point docStart = editor.posToDoc(getSelectionStart());
    	Point docEnd = editor.posToDoc(getSelectionEnd());
    	rc.addRedo(editor.clearSelection());
    	rc.addRedo(editor.getSourceModel().deleteRange(docStart, docEnd));
    	rc.setName("selectionDelete");

		if( editor != null ) {
    		editor.repaint();
		}
    	return rc;
    }

	public SourceCodeSelection copy() {
		LineSelection rc = null;
    	SourceCodeEditor editor = m_editor;
		if( editor != null ) {
			rc = new LineSelection(getSelectionStart(), getSelectionEnd(), editor.getText(getSelectionStart(), getSelectionEnd()));
		} else {
			rc = new LineSelection(getSelectionStart(), getSelectionEnd(), m_copyData);
		}
		return rc;
	}
	
///////////////////////////////////////////////////////////////////////////////
//	ClipboardOwner
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
	}
//	ClipboardOwner
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	Transferable
	public Object getTransferData(DataFlavor flavor) 
		throws UnsupportedFlavorException, IOException {
		Object rc = null;
		if( flavor.getRepresentationClass().isInstance(this)) {
			rc = this;
		} else {
			m_stringSelection = new StringSelection(getSelectedText());
			rc = m_stringSelection.getTransferData(flavor);
		}
		return rc;
	}
	
	private StringSelection m_stringSelection = new StringSelection("");
	private static DataFlavor[] availableFlavors = null;		
	public DataFlavor[] getTransferDataFlavors() {
		if( availableFlavors == null ) {
			DataFlavor[] ss = m_stringSelection.getTransferDataFlavors();
			availableFlavors = new DataFlavor[ss.length+1];
			for( int i=0; i < ss.length; i++ ) {
				availableFlavors[i] = ss[i];
			}
			availableFlavors[ss.length] = new DataFlavor(SourceCodeSelection.class, "Bounded source code selection");
		}
		return (DataFlavor[])availableFlavors.clone();
	} 
	
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		boolean rc = false;
		DataFlavor[] df = getTransferDataFlavors();
		for( int i=0; i<df.length; i++ ) {
			if( df[i].equals(flavor) ) {
				rc = true;
				break;
			}
		}
		return rc;
	}
//	Transferable	
///////////////////////////////////////////////////////////////////////////////
    public String getSelectedText() {
    	SourceCodeEditor editor = m_editor;
    	if( editor == null ) {
    		return m_copyData;
    	} else {
    		return editor.getText(getSelectionStart(), getSelectionEnd());
    	}
    }

    public UndoCommand toUppercase() {
    	UndoChain rc = new UndoChain("toUppercase");
    	SourceCodeEditor editor = m_editor;
    	boolean swap = false;
    	if( ( getSelectionStart().y > getSelectionEnd().y ) || 
    	    ( ( getSelectionStart().y == getSelectionEnd().y ) && ( getSelectionStart().x > getSelectionEnd().x ) ) ) {
    	    swapAnchor();
    	    swap = true;
    	}
		String str = editor.getText(getSelectionStart(), getSelectionEnd());
		String newStr = str.toUpperCase();
		
		rc.addRedo(editor.clearSelection());		
		rc.addRedo(editor.deleteRange(getSelectionStart(), getSelectionEnd()));
		
		Point docDest = editor.posToDoc(editor.getCaretPosition());
    	rc.addRedo(editor.getSourceModel().insertText(newStr, docDest));
    	
    	editor.setCaretPosition(getSelectionStart(), true);
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.LINE));
    	rc.addRedo(editor.setSelectionPosition(getSelectionEnd()));
    	if( swap ) {
    		editor.getEditCommands().selectSwapAnchor();
    	}
    	return rc;
    }
    
    public UndoCommand toLowercase() {
    	UndoChain rc = new UndoChain("toLowercase");
    	SourceCodeEditor editor = m_editor;
    	boolean swap = false;
    	if( ( getSelectionStart().y > getSelectionEnd().y ) || 
    	    ( ( getSelectionStart().y == getSelectionEnd().y ) && ( getSelectionStart().x > getSelectionEnd().x ) ) ) {
    	    swapAnchor();
    	    swap = true;
    	}
		String str = editor.getText(getSelectionStart(), getSelectionEnd());
		String newStr = str.toLowerCase();
		
		rc.addRedo(editor.clearSelection());		
		rc.addRedo(editor.deleteRange(getSelectionStart(), getSelectionEnd()));
		
		Point docDest = editor.posToDoc(editor.getCaretPosition());
    	rc.addRedo(editor.getSourceModel().insertText(newStr, docDest));
    	
    	editor.setCaretPosition(getSelectionStart(), true);
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.LINE));
    	rc.addRedo(editor.setSelectionPosition(getSelectionEnd()));
     	if( swap ) {
    		editor.getEditCommands().selectSwapAnchor();
    	}
    	return rc;
   }
    
    public UndoCommand toggleCase() {
    	UndoChain rc = new UndoChain("toggleCase");
    	SourceCodeEditor editor = m_editor;
    	boolean swap = false;
    	if( ( getSelectionStart().y > getSelectionEnd().y ) || 
    	    ( ( getSelectionStart().y == getSelectionEnd().y ) && ( getSelectionStart().x > getSelectionEnd().x ) ) ) {
    	    swapAnchor();
    	    swap = true;
    	}
		String str = editor.getText(getSelectionStart(), getSelectionEnd());
		
		StringBuffer newStr = new StringBuffer();
		for( int i=0; i<str.length(); i++ ) {
			if( Character.isLetter(str.charAt(i)) ) {
				if( Character.isUpperCase(str.charAt(i)) ) {
					newStr.append(Character.toLowerCase(str.charAt(i)));
				} else {
					newStr.append(Character.toUpperCase(str.charAt(i)));
				}
			} else {
				newStr.append(str.charAt(i));
			}
		}
		
		rc.addRedo(editor.clearSelection());		
		rc.addRedo(editor.deleteRange(getSelectionStart(), getSelectionEnd()));
		
		Point docDest = editor.posToDoc(editor.getCaretPosition());
    	rc.addRedo(editor.getSourceModel().insertText(newStr.toString(), docDest));
    	
    	editor.setCaretPosition(getSelectionStart(), true);
    	rc.addRedo(editor.setSelectionMode(SourceCodeSelection.LINE));
    	rc.addRedo(editor.setSelectionPosition(getSelectionEnd()));
    	if( swap ) {
    		editor.getEditCommands().selectSwapAnchor();
    	}
    	return rc;
    }
    
    private void validateSelection() {
    	SourceCodeEditor editor = m_editor;
    	if( m_posStart.equals(m_posEnd) ) {
    		if( editor != null && editor.getSelection()==this ) {
    			editor.clearSelection();
    		} else if( getParent() != null ) {
    			Container p = getParent();
    			p.remove(this);
    			Rectangle b = getBounds();
    			p.repaint(b.x, b.y, b.width, b.height);
    		}
    	}
    }
    
    private void cleanup() {
    	removeHierarchyListener(this);
    	m_dragGestureRecognizer.removeDragGestureListener(this);
    	m_dragGestureRecognizer = null;
    	m_dragSource = null;
    	m_editor = null;
    }
    
///////////////////////////////////////////////////////////////////////////////
//	DragGestureListener
    public void dragGestureRecognized(DragGestureEvent dge) {
		m_stringSelection = new StringSelection(getSelectedText());
    	if( (dge.getDragAction()&DnDConstants.ACTION_COPY) != 0 ) {
        	DragSource.getDefaultDragSource().startDrag(dge, DragSource.DefaultCopyDrop, this, this);
    	} else if( (dge.getDragAction()&DnDConstants.ACTION_MOVE) != 0 ) {
	        DragSource.getDefaultDragSource().startDrag(dge, DragSource.DefaultMoveDrop, this, this);
    	}    	
    }
//	DragGestureListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	DragSourceListener
    public void dragEnter(DragSourceDragEvent dsde) {
    }
    public void dragOver(DragSourceDragEvent dsde) {
    }
    public void dropActionChanged(DragSourceDragEvent dsde) {
    }
    public void dragExit(DragSourceEvent dse) {
    }
    public void dragDropEnd(DragSourceDropEvent dsde) {
    	if( dsde.getDropSuccess() ){
			DragAndDropCommand undoCmd = new DragAndDropCommand("dragAndDrop", this);
    		SourceCodeEditor editor = m_editor;
        	SelectionCommand selCmd = new SelectionCommand("dragAndDrop");
        	undoCmd.addRedo(selCmd);
        	selCmd.initSelection(editor);
    		m_copyData = getSelectedText();
			// creating this undo command with this allows the drag and drop commands to be combined if necessary.
			if( dsde.getDropAction()==DnDConstants.ACTION_MOVE ) {
				UndoCommand delCmd = delete();
        		undoCmd.addRedo(delCmd);
        	}
        	if( editor != null ) {
        		editor.clearSelection();
        	}
        	selCmd.finalSelection(editor);
        	editor.getUndoManager().addUndo(undoCmd);
    	}
    }
//	DragSourceListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ReferencePointListener
	public void pointChanged( Point p, String name ) {
    	SourceCodeEditor editor = m_editor;
		if( name.equals(m_startId) ) {
			m_posStart = editor.docToPos(p);
			adjustPositions();
			updateBounds();
		} else if( name.equals(m_endId ) ) {
			m_posEnd = editor.docToPos(p);
			adjustPositions();
			updateBounds();
		}
		validateSelection();
	}
	
	public String changeReferenceName( String name ) {
    	SourceCodeEditor editor = m_editor;
		if( name.equals(m_startId) ) {
			m_startId = "LINE_START_"+editor.getUnusedReferenceId();
			return m_startId;
		} else if( name.equals(m_endId ) ) {
			m_endId = "LINE_END_"+editor.getUnusedReferenceId();
			return m_endId;
		}
		return "";
	}
//	ReferencePointListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	HierarchyListener
	public void hierarchyChanged(HierarchyEvent e) {
    	SourceCodeEditor editor = m_editor;
		if( (e.getChangeFlags()&HierarchyEvent.PARENT_CHANGED) != 0 ) {
			if( e.getChanged() == this ) {
				if( getParent() != null ) {
	    			updateBounds();
					editor.addReferencePoint(editor.posToDoc(m_posStart), m_startId, this);
					editor.addReferencePoint(editor.posToDoc(m_posEnd), m_endId, this);
				} else {
					editor.removeReferencePoint(m_startId, this);
					editor.removeReferencePoint(m_endId, this);
					cleanup();
				}
			}
		}
	}
//	HierarchyListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	Autoscroll
	public Insets getAutoscrollInsets() {
		return new Insets(getHeight()/2, getWidth()/2, getHeight()/2, getWidth()/2);
	}
	
	public void autoscroll(Point cursorLocn) {
		if( ( getParent() != null ) && ( getParent() instanceof Autoscroll ) ) {
			cursorLocn = SwingUtilities.convertPoint(this, cursorLocn, getParent());
			Rectangle b = getParent().getBounds();
			Insets asi = ((Autoscroll)getParent()).getAutoscrollInsets();
			if( ( cursorLocn.y < asi.top ) || 
				( cursorLocn.y > b.height-asi.bottom ) ||
				( cursorLocn.x < asi.left ) ||
				( cursorLocn.x > b.width-asi.right ) ) {
//				System.err.println("Autoscrolling parent");
				((Autoscroll)getParent()).autoscroll(cursorLocn);
			}
		}
	}
//	Autoscroll
///////////////////////////////////////////////////////////////////////////////
}