/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.undo;

import gvid.undo.*;
import gvid.document.*;
import java.awt.Point;
import gvid.ui.editor.SourceCodeEditor;

public class DeleteLineRangeCommand 
	implements UndoCommand {
	private String m_name = "";	
	private SourceCodeModel m_model = null;
	private int m_row = 0;
	private int m_colStart = 0;
	private int m_colEnd = 0;
	private String m_str = "";

	public DeleteLineRangeCommand( String name, SourceCodeModel model, int row, int start, int end, String str ) {
		m_name = name;
		m_model = model;
		m_row = row;
		m_colStart = start;
		m_colEnd = end;
		m_str = str;
	}

    public String getName() {
    	return m_name;
    }

    public void setName( String name ) {
    	m_name = name;
    }

    public boolean isEditCmd() {
    	return true;
    }

    public void undo(SourceCodeEditor currentEditor) {
    	m_model.insertText(m_str, new Point(Math.min(m_colStart, m_colEnd), m_row));
    	currentEditor.setCaretPosition(currentEditor.docToPos(new Point(m_colStart, m_row)), false);
    	currentEditor.repaint();
    }

    public void redo(SourceCodeEditor currentEditor) {
    	currentEditor.setCaretPosition(currentEditor.docToPos(new Point(m_colStart, m_row)), false);
    	m_model.deleteRange(new Point(m_colStart, m_row), new Point(m_colEnd, m_row));
    	currentEditor.repaint();
    }
    
    public boolean combineCommands( UndoCommand cmd ) {
    	boolean rc = false;
    	return rc;
    }
}