/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.editor.undo;

import gvid.undo.*;
import java.awt.Point;
import gvid.document.SourceCodeModel;
import gvid.ui.editor.SourceCodeEditor;

public abstract class JoinSplitLineBase 
	implements gvid.undo.UndoCommand {
	private String m_name = "";
	private SourceCodeModel m_model = null;
	private Point m_pos = new Point();
	private int m_count = 1;
	
	public JoinSplitLineBase( String name, SourceCodeModel model, Point p ) {
		m_name = name;
		m_model = model;
		m_pos.x = p.x;
		m_pos.y = p.y;
	}
		
    public String getName() {
    	return m_name;
    }
    
    public void setName( String name ) {
    	m_name = name;
    }
    
    public boolean isEditCmd() {
    	return true;
    }
    
    protected void splitLine() {
		for( int i = 0; i < m_count; i++ ) {
    		m_model.split( m_pos );
    	}
    }
    
    protected void joinLine() {
		for( int i = 0; i < m_count; i++ ) {
	   		m_model.joinLine( m_pos.y );
	   	}
    }

    public boolean combineCommands( UndoCommand cmd ) {
    	boolean rc = false;
    	if( getClass().isInstance(cmd) ) {
    		m_count += ((JoinSplitLineBase)cmd).m_count;
    		rc = true;
    	}
    	return rc;
    }
}