/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.menu;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import gvid.ui.MainFrame;

public class DocumentMenuItem 
	extends 	javax.swing.JMenuItem
	implements	HierarchyListener,
				ContainerListener, 
				PropertyChangeListener,
				ActionListener,
				MouseListener {
		
	JInternalFrame m_frame = null;
	int m_docMenuStart = 0;
	
	public DocumentMenuItem(JInternalFrame frame, int indexStart ) {
		super((indexStart+1)+" "+frame.getTitle());
		m_frame = frame;
		m_docMenuStart = indexStart;
		addHierarchyListener(this);
		addActionListener(this);
		addMouseListener(this);
	}
	
	public JInternalFrame getFrame() {
		return m_frame;
	}
	
	private void updateTitle() {
		// Find the index of this menu item
		if( getParent() != null ) {
			Component[] comps = getParent().getComponents();
			int index = m_docMenuStart;
			for( ; index < comps.length; index++ ) {
				if( comps[index] == this ) {
					break;
				}
			}
    		index = index-m_docMenuStart+1;
    		String indexStr = ""+index;
    		setText(indexStr+" "+m_frame.getTitle());
    		setMnemonic(indexStr.charAt(indexStr.length()-1));
		}
	}
///////////////////////////////////////////////////////////////////////////////
//	HierarchyListener
    public void hierarchyChanged(HierarchyEvent e) {
    	if( (e.getChangeFlags() & HierarchyEvent.PARENT_CHANGED) != 0 && 
    		e.getChanged() == this ) {
    		// the parent has changed
    		if( getParent() == null ) {
    			// the item was removed - clean up
    			m_frame.removePropertyChangeListener("title", this);
    		} else {
    			// the item was added - setup
    			m_frame.addPropertyChangeListener("title", this);
    			updateTitle();
    			getParent().addContainerListener(this);
    		}
    	}
//    	System.err.println(e);
    }
//	HierarchyListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	PropertyChangeListener
	public void propertyChange(PropertyChangeEvent evt) {
		updateTitle();
	}
//	PropertyChangeListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ContainerListener
	public void componentAdded(ContainerEvent e) {
    	updateTitle();
	}
	
	public void componentRemoved(ContainerEvent e) {
		if( e.getChild() == this ) {
    		((Container)(e.getComponent())).removeContainerListener(this);
		} else {
    		updateTitle();
		}
	}
//	ContainerListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	ActionListener
	public void actionPerformed(ActionEvent evt) {
		try {
			m_frame.setSelected(true);
		} catch( PropertyVetoException e ) {
			System.err.println(e);
		}
	}
//	ActionListener
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//	MouseListener
    public void mouseClicked(MouseEvent e) {
    }
    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    	MainFrame.getMainFrame().getStatusSource().setStatusMsg("Activates this window");
    }
    public void mouseExited(MouseEvent e) {
    	MainFrame.getMainFrame().getStatusSource().setStatusMsg("");
    }
//	MouseListener
///////////////////////////////////////////////////////////////////////////////

}