/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.menu;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import gvid.command.*;

public class ViDMenuBar 
	extends javax.swing.JMenuBar
	implements CommandGroupListener {
		
	JMenu m_fileMenu = new JMenu("File");
	JMenu m_editMenu = new JMenu("Edit");
	JMenu m_viewMenu = new JMenu("View");
	JMenu m_projMenu = new JMenu("Project");
	WindowMenu m_windowMenu = null;
	JMenu m_helpMenu = new JMenu("Help");

	public ViDMenuBar(CommandHandler cmdHandler, JDesktopPane desktop) {		
		setBorder( new gvid.ui.border.HalfNarrowBevelBorder() );
		
		m_fileMenu.setMnemonic('F');
		{
			m_fileMenu.add(new MenuCommand("New File", 'N', CommandBase.FILEGROUP, FileCommands.NEWTEXT, cmdHandler, "Creates a new document" ));
			m_fileMenu.add(new MenuCommand("Open...", 'O', CommandBase.FILEGROUP, FileCommands.FILEOPEN, cmdHandler, "Opens and existing document" ));
			m_fileMenu.add(new FileMenuCommand("Close", 'C', CommandBase.FILEGROUP, FileCommands.FILECLOSE, cmdHandler, "Closes the document", desktop ));
			m_fileMenu.addSeparator();
			m_fileMenu.add(new MenuCommand("New Project", 'R', CommandBase.FILEGROUP, FileCommands.PROJECTNEW, cmdHandler, "Creates a new project" ));
			m_fileMenu.add(new MenuCommand("Open Project", 'J', CommandBase.FILEGROUP, FileCommands.PROJECTOPEN, cmdHandler, "Open a project" ));
			m_fileMenu.add(new MenuCommand("Save Project", 'V', CommandBase.FILEGROUP, FileCommands.PROJECTSAVE, cmdHandler, "Saves the project" ));
			m_fileMenu.add(new MenuCommand("Close Project", 'T', CommandBase.FILEGROUP, FileCommands.PROJECTCLOSE, cmdHandler, "Closes the project" ));
			m_fileMenu.addSeparator();
			m_fileMenu.add(new FileMenuCommand("Save", 'S', CommandBase.FILEGROUP, FileCommands.FILESAVE, cmdHandler, "Saves the document", desktop ));
			m_fileMenu.add(new MenuCommand("Save As...", 'A', CommandBase.FILEGROUP, FileCommands.FILESAVEAS, cmdHandler, "Saves the document with a new name" ));
			m_fileMenu.add(new MenuCommand("Save All", 'L', CommandBase.FILEGROUP, FileCommands.FILESAVEALL, cmdHandler, "Saves all open documents" ));
/*			
			m_fileMenu.addSeparator();
			m_fileMenu.add(new MenuCommand("Page Setup", 'N', cmdHandler ));
			m_fileMenu.add(new MenuCommand("Print", 'N', cmdHandler ));
			m_fileMenu.addSeparator();
			m_fileMenu.add(new JMenu("Recent Files", cmdHandler ));
*/			
			m_fileMenu.addSeparator();
			m_fileMenu.add(new MenuCommand("Exit", 'X', CommandBase.FILEGROUP, FileCommands.APPLICATIONEXIT, cmdHandler, "Quits the application; Prompts to save documents" ));
		}
		add(m_fileMenu);
		
		m_editMenu.setMnemonic('E');
		{
			m_editMenu.add( new MenuCommand("Undo", 'U', CommandBase.EDITGROUP, EditCommands.UNDO, cmdHandler, "Undoes the last action" ));
			m_editMenu.add( new MenuCommand("Redo", 'R', CommandBase.EDITGROUP, EditCommands.REDO, cmdHandler, "Redoes the previously undone action" ));
			m_editMenu.addSeparator();
			m_editMenu.add( new MenuCommand("Cut", 'T', CommandBase.EDITGROUP, EditCommands.CUT, cmdHandler, "Removes the selection and moves it to the clipboard" ));
			m_editMenu.add( new MenuCommand("Copy", 'C', CommandBase.EDITGROUP, EditCommands.COPY, cmdHandler, "Copies the selection to the clipboard" ));
			m_editMenu.add( new MenuCommand("Paste", 'P', CommandBase.EDITGROUP, EditCommands.PASTE, cmdHandler, "Inserts the clipboard contents at the insertion point" ));
			m_editMenu.add( new MenuCommand("Delete", 'D', CommandBase.EDITGROUP, EditCommands.DELETE, cmdHandler, "Removes the selection" ));
			m_editMenu.addSeparator();
			m_editMenu.add( new MenuCommand("Select All", 'S', CommandBase.EDITGROUP, EditCommands.SELECTALL, cmdHandler, "Selects the entire document" ));
			m_editMenu.addSeparator();
			m_editMenu.add( new MenuCommand("Find", 'F', CommandBase.EDITGROUP, EditCommands.FIND, cmdHandler, "Finds the specified text" ));
//			JMenu advancedMenu = new JMenu("Advanced");
//			advancedMenu.add( new MenuCommand("Indent Selection", '', CommandBase.EDITGROUP, EditCommands.SELECTIONINDENT
		}
		add(m_editMenu);
		
		m_viewMenu.setMnemonic('V');
		{
			m_viewMenu.add( new MenuCommand("Project Window", 'P', CommandBase.VIEWGROUP, ViewCommands.TOGGLEPROJECTVIEW, cmdHandler, "Show the project window" ));
		}
		add(m_viewMenu);
		
		m_projMenu.setMnemonic('P');
		{
			m_projMenu.add( new MenuCommand("Build Project", 'B', CommandBase.PROJECTGROUP, ProjectCommands.BUILD, cmdHandler, "Builds the current project" ) );
			m_projMenu.add( new MenuCommand("Compile File", 'C', CommandBase.PROJECTGROUP, ProjectCommands.COMPILEFILE, cmdHandler, "Compiles the active file" ));
			m_projMenu.addSeparator();
			m_projMenu.add( new MenuCommand("Rebuild All", 'A', CommandBase.PROJECTGROUP, ProjectCommands.REBUILDALL, cmdHandler, "Rebuilds all files in the current project" ) );
			m_projMenu.addSeparator();
			m_projMenu.add( new MenuCommand("Settings", 'S', CommandBase.PROJECTGROUP, ProjectCommands.PROJECTSETTINGS, cmdHandler, "Edit the project settings" ));
		}
		add(m_projMenu);
		
		m_windowMenu = new WindowMenu(desktop);
		m_windowMenu.setMnemonic('W');
		{
//			m_windowMenu.add(new MenuCommand("New Window", 'N', cmdHandler ));
//			m_windowMenu.add(new MenuCommand("Split", 'P', cmdHandler ));
//			m_windowMenu.add(new MenuCommand("Close", 'O', cmdHandler ));
			m_windowMenu.add(new MenuCommand("Close All", 'L', CommandBase.FILEGROUP, FileCommands.WINDOWCLOSEALL, cmdHandler, "Closes all open windows" ));
			m_windowMenu.addSeparator();
			m_windowMenu.add(new MenuCommand("Next", 'X', CommandBase.FILEGROUP, FileCommands.WINDOWNEXT, cmdHandler, "Activates the next undocked window" ));
			m_windowMenu.add(new MenuCommand("Prev", 'V', CommandBase.FILEGROUP, FileCommands.WINDOWPREV, cmdHandler, "Activates the previous undocked window" ));
			m_windowMenu.addSeparator();
			m_windowMenu.add(new MenuCommand("Cascade", 'C', CommandBase.FILEGROUP, FileCommands.WINDOWCASCADE, cmdHandler, "Arranges the windows as overlapping tiles" ));
			m_windowMenu.add(new MenuCommand("Tile Horizontally", 'H', CommandBase.FILEGROUP, FileCommands.WINDOWTILEHORIZONTAL, cmdHandler, "Arranges the windows as horizontal, nonoverlapping tiles" ));
			m_windowMenu.add(new MenuCommand("Tile Vertically", 'T', CommandBase.FILEGROUP, FileCommands.WINDOWTILEVERTICAL, cmdHandler, "Arranges the windows as vertical, nonoverlapping tiles" ));
			m_windowMenu.addSeparator();
			m_windowMenu.setDocumentMenuStart(m_windowMenu.getItemCount());
			m_windowMenu.add(new MenuCommand("Windows...", 'W', CommandBase.FILEGROUP, FileCommands.WINDOWLIST, cmdHandler, "Manages the currently open windows" ));
		}
		add(m_windowMenu);
		
		m_helpMenu.setMnemonic('H');
		{
			m_helpMenu.add(new JMenuItem("Command List", 'C'));
			m_helpMenu.add(new JMenuItem("About ViD", 'A'));
		}
		add(m_helpMenu);

		m_fileMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.FILEGROUP));
		m_editMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.EDITGROUP));
		m_viewMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.VIEWGROUP));
		m_projMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.PROJECTGROUP));
		m_windowMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.WINDOWGROUP));
		m_helpMenu.setVisible(cmdHandler.isGroupAvailable(CommandBase.HELPGROUP));
		
		cmdHandler.addCommandGroupListener(this);
}
///////////////////////////////////////////////////////////////////////////////
// CommandGroupListener 
	public void groupAvailabilityChanged( int group, boolean newValue, boolean oldValue ) {
		JMenu menu = null;
		switch( group ) {
			case CommandBase.FILEGROUP: {
				menu = m_fileMenu;
			} break;
			case CommandBase.EDITGROUP: {
				menu = m_editMenu;
			} break;
			case CommandBase.VIEWGROUP: {
				menu = m_viewMenu;
			} break;
			case CommandBase.PROJECTGROUP: {
				menu = m_projMenu;
			} break;
			case CommandBase.TOOLGROUP: {
//				menu = m_toolMenu;
			} break;
			case CommandBase.WINDOWGROUP: {
				menu = m_windowMenu;
			} break;
			case CommandBase.HELPGROUP: {
				menu = m_helpMenu;
			} break;
		}
		if( menu != null ) {
			menu.setVisible(newValue);
		}
	}
// CommandGroupListener 
///////////////////////////////////////////////////////////////////////////////
	//{{DECLARE_CONTROLS
	//}}
}