/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.menu;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class WindowMenu 
	extends 	javax.swing.JMenu 
	implements	ContainerListener {
	private JDesktopPane m_desktop = null;
	private int m_docIndexStart = 0;
	private int m_docCount = 0;
	private Hashtable m_windowMenuMap = new Hashtable();
					
	public WindowMenu(JDesktopPane desktop)	{
		super("Window");
		setMnemonic('W');
		m_desktop = desktop;
		m_desktop.addContainerListener(this);
	}
	
	public void setDocumentMenuStart( int docIndexStart ) {
		m_docIndexStart = docIndexStart;
		// TODO - handle case where document menus are already present.
		// move document menus to new location
	}
	
	public void addDocumentMenu(JInternalFrame frame) {
		if( m_docCount >= 10 ) {
    		m_docCount--;
    		DocumentMenuItem oldDocMenu = (DocumentMenuItem)getMenuComponent(m_docIndexStart+m_docCount);    			
    		remove(m_docIndexStart+m_docCount);
    		m_windowMenuMap.remove(oldDocMenu.getFrame());
		}
    	JMenuItem docMenu = new DocumentMenuItem(frame, m_docIndexStart);
    	insert(docMenu, m_docIndexStart+m_desktop.getIndexOf(frame));
    	m_windowMenuMap.put( frame, docMenu );
    	m_docCount++;
	}
	
	public void removeDocumentMenu(JInternalFrame frame) {
    	JMenuItem docMenu = (JMenuItem)m_windowMenuMap.remove(frame);
    	if( docMenu != null ) {
    		remove(docMenu);
    		m_docCount--;
    	}
    	JInternalFrame[] frames = m_desktop.getAllFrames();
    	if( m_docCount < frames.length ) {
    		docMenu = new DocumentMenuItem(frames[m_docCount], m_docIndexStart);
    		insert(docMenu, m_docIndexStart+m_docCount);
    		m_windowMenuMap.put( frames[m_docCount], docMenu );
    		m_docCount++;
    	}
	}
	
///////////////////////////////////////////////////////////////////////////////
//	ContainerListener
    public void componentAdded(ContainerEvent e) { 
    	if( e.getChild() instanceof JInternalFrame ) {
    		addDocumentMenu( (JInternalFrame)(e.getChild()) );
    	}
    }
    
    public void componentRemoved(ContainerEvent e) { 
    	if( e.getChild() instanceof JInternalFrame ) {
    		removeDocumentMenu( (JInternalFrame)(e.getChild()) );
    	}
    }
//	ContainerListener
///////////////////////////////////////////////////////////////////////////////
}