/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.splitscrollpane;

import javax.swing.*;
import java.beans.*;
import java.awt.*;

public class SplitMover extends javax.swing.JComponent
{
    private SplitControl m_splitControl = null;
    private int m_position = 0;
    
	public SplitMover(SplitControl parent) {
	    setOpaque(false);
	    m_splitControl = parent;
	}
	
	public int getPosition() {
	    return m_position;
	}
	
	public void setPosition( int position ) {
	    m_position = position;
	    if( m_position <= m_splitControl.getInactivePosition() ) {
	        m_position = m_splitControl.getInactivePosition() - 1;
	    }
	    if( m_position >= m_splitControl.getMaxPosition() ) {
	        m_position = m_splitControl.getMaxPosition() + 2;
	    }
	}
	
	public void move( int distance ) {
	    setPosition(m_position + distance);
	    updateBounds();
	}
	
	public void updateBounds() {
	    if( getParent() != null ) {
	        Insets insets = getParent().getInsets();
	        Dimension size = getParent().getSize();
            if( m_splitControl.getAlignment() == SwingConstants.HORIZONTAL ) {
	            setBounds(insets.left+m_position+1, m_splitControl.getActiveEdge(), SplitControl.DEFAULTSIZE-3, size.height-insets.top-insets.bottom);
            } else {
	            setBounds(m_splitControl.getActiveEdge(), insets.top+m_position+1, size.width-insets.left-insets.right, SplitControl.DEFAULTSIZE-3);
            }
	    }
	}
	
	public void paint( Graphics g ) {
	    g.setColor(Color.black);
	    g.setXORMode(Color.white);
	    Rectangle b = getBounds();
	    b.x = b.y = 0;
	    ditherArea(b, g);
	}
	
	private Image m_ditherImage = null;
	private Rectangle m_lastImageSize = null;
	private void ditherArea( Rectangle b, Graphics g ) {
	    if( m_ditherImage == null || !m_lastImageSize.equals(b) ) {
	        m_lastImageSize = new Rectangle(b);
	        m_ditherImage = createImage(b.width, b.height+1);
	        Graphics g2 = m_ditherImage.getGraphics();
	        g2.setColor(Color.white);
	        g2.fillRect(0, 0, b.width, b.height);
	        g2.setColor(Color.black);
	        for( int x=b.x-b.height-1-(b.height%2); x<b.x+b.width+1; x+=2 ) {
	            g2.drawLine(x,0,x+b.height+1,b.height+1);
	        }
	    }
	    
	    if( (getBounds().x+getBounds().y+b.x+b.y) % 2 == 0 ) {
	        g.drawImage(m_ditherImage, b.x, b.y, b.x+b.width, b.y+b.height, 0, 1, b.width, b.height+1, null);
	    } else {
	        g.drawImage(m_ditherImage, b.x, b.y, null);
	    }
	}
}