/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.ui.status;

import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.awt.*;
import java.util.*;

public class StatusPanel 
	extends 	javax.swing.JPanel
	implements  StatusListener {
		
	private StatusSource m_status = null;
	private JLabel m_statusLabel = new JLabel("Ready");
	private JPanel m_customStatus = new JPanel();
	private DefaultStatus m_defaultStatus = new DefaultStatus();
	
	public StatusPanel() {
//		setFont(new Font("SanSerif", Font.PLAIN, 12));
		m_statusLabel.setForeground(Color.black);
//		m_statusLabel.setFont(getFont());
		m_statusLabel.setText("Ready");
		
		m_customStatus.setLayout(new FlowLayout(FlowLayout.RIGHT, 2, 0));
		
		add(m_statusLabel);
		setBorder(new EmptyBorder(2,1,0,0));
		setLayout(new BorderLayout() );
		add( m_statusLabel, BorderLayout.WEST );
		add( m_customStatus, BorderLayout.EAST );
		setStatusSource(m_defaultStatus);
	}
	
	public StatusSource getStatusSource() {
		return m_status;
	}
	
	public void setStatusSource( StatusSource src ) {
		if( m_status != null ) {
			m_status.removeStatusListener(this);
			unbuildStatus();
		}
		if( src == null ) {
			src = m_defaultStatus;
		}
		m_status = src;
		if( m_status != null ) {
			buildStatus();
			m_status.addStatusListener(this);
		}
	}
	
	private void buildStatus() {
		if( m_status != null ) {
			m_statusLabel.setText(m_status.getStatusMsg());
			for( int i=0; i<m_status.getStatusCount(); i++ ) {
				JPanel panel = new JPanel();
				panel.setBorder(new gvid.ui.border.NarrowBevelBorder());
				panel.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
				
				JLabel label = new JLabel( m_status.getStatusString(i) );
				label.setHorizontalAlignment(SwingConstants.CENTER);
//				label.setFont(getFont());
				if( m_status.isStatusActive(i) ) {
					label.setForeground(Color.black);
				} else {
					label.setForeground(Color.gray);
				}
				panel.add(label);
				m_customStatus.add(panel);
			}
		}
		doLayout();
	}
	
	private void unbuildStatus() {
		m_statusLabel.setText("Ready");
		
		m_customStatus.removeAll();

		doLayout();
	}
///////////////////////////////////////////////////////////////////////////////
//	StatusListener
	public void statusMessageChanged() {
		if( m_status != null ) {
			m_statusLabel.setText(m_status.getStatusMsg());
		}
		validate();
	}
	
	public void statusChanged( int index, String str, boolean active ) {
		Component label = ((Container)m_customStatus.getComponent(index)).getComponent(0);
		if( label instanceof JLabel ) {
			((JLabel)label).setText(str);
		}
		if( active ) {
			label.setForeground(Color.black);
		} else {
			label.setForeground(Color.gray);
		}
		validate();
	}
	
	public void statusChanged() {
		unbuildStatus();
		buildStatus();
		doLayout();
	}
//	StatusListener
///////////////////////////////////////////////////////////////////////////////

	private class DefaultStatus 
		implements StatusSource {
		Vector m_listeners = new Vector();
		
		public void addStatusListener( StatusListener l ) {
			m_listeners.addElement(l);
		}
		
		public void removeStatusListener( StatusListener l ) {
			m_listeners.removeElement(l);
		}
		
		String m_message = "Ready";
		public String getStatusMsg() {
			return m_message;
		}
		public void setStatusMsg( String str ) {
			if( str.length() == 0 ) {
				str = " ";
			}
			if( !str.equals(m_message) ) {
				m_message = str;
				Enumeration enum = m_listeners.elements();
				while( enum.hasMoreElements() ) {
					StatusListener l = (StatusListener)enum.nextElement();
					l.statusMessageChanged();
				}
			}
		}
		
		public int getStatusCount() {
			return 0;
		}
		public String getStatusString(int index) {
			return "";
		}
		public boolean isStatusActive(int index) {
			return false;
		}

	}
}