/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.util;

import java.util.*;

public class NotifyingList 
	implements	java.util.List,
				ListListener,
				ListIteratorListener {
	private List m_list = null;
	
	public NotifyingList() {
	}
	
	public NotifyingList( List list ) {
		m_list = list;
	}
	
	public void setList( List list ) {
		List oldList = m_list;
		m_list = list;
		if( oldList != list ) {
			notifyListChanged();
		}
	}
	
	private Vector m_listeners = new Vector();
	public void addListListener( ListListener l ) {
		if( l != this ) {
			m_listeners.add(l);
		}
	}
	
	public void removeListListener( ListListener l ) {
		m_listeners.remove(l);
	}
	
	private void notifyListChanged() {
		Iterator iter = m_listeners.iterator();
		while( iter.hasNext() ) {
			ListListener l = (ListListener)iter.next();
			l.listChanged(this);
		}
	}
	
	public void listChanged(NotifyingList list) {
		if( list != this ) {
			notifyListChanged();
		}
	}
	
	public void listChanged( NotifyingListIterator list ) {
		notifyListChanged();
	}
	
	public void listIteratorChanged(NotifyingListIterator iter) {
		notifyListChanged();
	}
	
    public boolean add(Object o) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.add(o);
			notifyListChanged();
		}
        return rc;
    }

    public void add(int index, Object element) {
		if( m_list != null ) {
			m_list.add(index, element);
			notifyListChanged();
		}
    }

    public boolean addAll(int index, Collection c) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.addAll(index, c);
			notifyListChanged();
		}
        return rc;
    }

    public boolean addAll(Collection c) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.addAll(c);
			notifyListChanged();
		}
        return rc;
    }

    public void clear() {
		if( m_list != null ) {
			m_list.clear();
			notifyListChanged();
		}
    }

    public boolean contains(Object o) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.contains(o);
		}
        return rc;
    }

    public boolean containsAll(Collection c) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.containsAll(c);
		}
        return rc;
    }

	// may not be associative - dangerous
    public boolean equals(Object o) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.equals(o);
		}
        return rc;
    }

    public Object get(int index) {
    	Object rc = null;
		if( m_list != null ) {
			rc = m_list.get(index);
		}
        return rc;
    }

    public int hashCode() {
    	int rc = 0;
		if( m_list != null ) {
			rc = m_list.hashCode();
		}
        return rc;
    }

    public int indexOf(Object o) {
    	int rc = -1;
		if( m_list != null ) {
			rc = m_list.indexOf(o);
		}
        return rc;
    }

    public boolean isEmpty() {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.isEmpty();
		}
        return rc;
    }

    public Iterator iterator() {
    	Iterator rc = null;
		if( m_list != null ) {
			rc = m_list.iterator();
		}
        return rc;
    }

    public int lastIndexOf(Object o) {
    	int rc = -1;
		if( m_list != null ) {
			rc = m_list.lastIndexOf(o);
		}
        return rc;
    }

    public ListIterator listIterator(int index) {
    	ListIterator rc = null;
		if( m_list != null ) {
			rc = m_list.listIterator(index);
			rc = new NotifyingListIterator(rc);
			// This references is one way so the NotifyingListIterator should 
			// be garbage collected once the caller is done with it.
			((NotifyingListIterator)rc).addListIteratorListener(this);
		}
        return rc;
    }

    public ListIterator listIterator() {
    	ListIterator rc = null;
		if( m_list != null ) {
			rc = m_list.listIterator();
			rc = new NotifyingListIterator(rc);
			// This references is one way so the NotifyingListIterator should 
			// be garbage collected once the caller is done with it.
			((NotifyingListIterator)rc).addListIteratorListener(this);
		}
        return rc;
    }

    public boolean remove(Object o) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.remove(o);
			notifyListChanged();
		}
        return rc;
    }

    public Object remove(int index) {
    	Object rc = null;
		if( m_list != null ) {
			rc = m_list.remove(index);
			notifyListChanged();
		}
        return rc;
    }

    public boolean removeAll(Collection c) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.removeAll(c);
			notifyListChanged();
		}
        return rc;
    }

    public boolean retainAll(Collection c) {
    	boolean rc = false;
		if( m_list != null ) {
			rc = m_list.retainAll(c);
			notifyListChanged();
		}
        return rc;
    }

    public Object set(int index, Object element) {
    	Object rc = null;
		if( m_list != null ) {
			rc = m_list.set(index, element);
			notifyListChanged();
		}
        return rc;
    }

    public int size() {
    	int rc = 0;
		if( m_list != null ) {
			rc = m_list.size();
		}
        return rc;
    }

    public List subList(int fromIndex, int toIndex) {
    	List rc = null;
		if( m_list != null ) {
			rc = m_list.subList(fromIndex, toIndex);
			rc = new NotifyingList(rc);
			// This references is one way so the NotifyingList should 
			// be garbage collected once the caller is done with it.
			((NotifyingList)rc).addListListener(this);
		}
        return rc;    
    }

    public Object[] toArray() {
    	Object[] rc = {};
		if( m_list != null ) {
			rc = m_list.toArray();
		}
        return rc;
    }

    public Object[] toArray(Object a[]) {
    	Object[] rc = {};
		if( m_list != null ) {
			rc = m_list.toArray(a);
		}
        return rc;
    }

}