/******************************************************************************
 * GViD Source Code Editor
 * Copyright (C) 2002  Alan Oursland
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Alan Oursland
 * alan@oursland.net
 *****************************************************************************/
package gvid.util;

import javax.swing.table.*;
import java.util.*;

public class TableListModel extends javax.swing.table.AbstractTableModel {
	private Vector m_listItems = new Vector();
	
    public Class getColumnClass(int columnIndex) {
        return String.class;
    }

    public int getColumnCount() {
        return 1;
    }

    public String getColumnName(int column) {
        return "Classpath";
    }

    public int getRowCount() {
        return m_listItems.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object rc = null;
        rc = m_listItems.get(rowIndex);
        return rc;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
    	m_listItems.set( rowIndex, aValue );
    	fireTableCellUpdated( rowIndex, columnIndex );
    }

    public void addItem() {
    	m_listItems.add("");
    	fireTableRowsInserted(getRowCount()-1, getRowCount()-1);
    }
    
    public void addAll( Collection c ) {
    	m_listItems.addAll(c);
    	fireTableRowsInserted( getRowCount()-c.size()-1, getRowCount());
    }
    
    public void clear() {
    	int count = getRowCount();
    	m_listItems.clear();
    	if( count > 0 ) {
    		fireTableRowsDeleted( 0, count-1 );
    	}
    }
    
    public void removeItem( int index ) {
    	m_listItems.remove(index);
    	fireTableRowsDeleted(index, index);
    }
    
    public void moveItemUp( int index ) {
    	if( index > 0 ) {
    		m_listItems.add( index-1, m_listItems.remove(index) );
    		fireTableRowsUpdated(index-1, index);
    	}
    }
    
    public void moveItemDown( int index ) {
    	if( index < getRowCount()-1 ) {
    		m_listItems.add( index+1, m_listItems.remove(index) );
    		fireTableRowsUpdated(index, index+1);    		
    	}
    }
    
    public boolean listEquals( List list ) {
    	return list.equals(m_listItems);
    }
    
    public void setList( List list ) {
    	list.clear();
    	list.addAll(m_listItems);
    }
}